# -*- encoding: utf-8 -*-

__author__ = 'nika'

from tree import Tree
from .node import Node
import sys, pickle, os
from xml.sax import make_parser, handler

class TreeParser(handler.ContentHandler):

    def __init__(self, path):
        self.tree = Tree(path)
        #self.current_node = None
        #self.current_tag = ""
        #self.nodes_probs = {}
        #self.nodes = {}
        #self.nodes_from_to = {}
        #self.productions = []
        #self.in_chosen_node = False
        #self.nid = None
        #self.in_chosen_children = False
        #self.current_prod = []
        #self.full_tree = False
        #self.centre = None
        self.children = []
        #self.od = None
        #self.do = None
        #self.chosen_tree = []
        #self.chosen_tree_dep = []
        #self.prob_tree = []
        #self.prob_tree_dep = []
        #self.precision = None
        #self.precision_dep = None
        #self.recall = None
        #self.recall_dep = None
        #self.chosen_tree_centres = {}
        #self.prob_tree_centres = {}
        #self.nodes_to_expand = []
        #self.amb_count = 0
        #self.is_amb = False
        #self.more_centres = []
        #self.current_arg = None
        #self.n_more_c = {}
        self.nodes_ext = {}
        self.current_prod = []
        self.in_terminal = False
        self.current_terminal = {}
        self.current_terminal_id = None

    def getTree(self):
            return self.tree

    def startElement(self, name, attrs):
        self.current_tag = name
        if name == "node":
            attr_dict = {}
            for name in attrs.getNames():
                attr_dict[name] = attrs.getValue(name)
            self.current_node = Node(attr_dict)
            if "chosen" in attrs.getNames() and attrs.getValue("chosen") == "true":
                self.tree.addChosenNode(self.current_node)
            else:
                self.tree.addNode(self.current_node)
        elif name == "children":
            pass
        elif name == "child":
            if attrs.getValue("head") == "true":
                self.centre = attrs.getValue("nid")
                self.tree.addHead(self.current_node.getID(), self.centre)
            else:
                self.children.append(attrs.getValue("nid"))
            self.current_prod.append(( attrs.getValue("nid"), attrs.getValue("head") ))
        elif name == "terminal":
            self.current_node.setTerminal()
            self.current_terminal = {}
            self.in_terminal = True
            self.current_terminal['id'] = int(self.current_node.getFrom())
            self.tree.addHead(self.current_node.getID(), self.current_node.getID())
        elif name == "nonterminal":
            self.current_node.setNonterminal()
        elif name == "f":
            self.current_arg = attrs.getValue("type")

    def endElement(self, name):                         # zlicza w zależności od węzła
        if name == "node":
            self.productions = []
            self.in_chosen_node = False
        if name == "children":
            if self.centre:
                self.current_node.addChildren( {'centre': self.centre, 'productions':self.children, 'whole_prod':self.current_prod} )
            else:
                self.current_node.addChildren( {'productions':self.children, 'whole_prod':self.current_prod} )
                """
                if len(self.children) == 1:
                    self.current_node.addChildren( {'centre': self.children[0], 'productions':[], 'whole_prod':self.current_prod} )
                else:
                    print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
                """
            self.tree.addParent(self.current_node, self.children + [self.centre])
            self.children = []
            self.in_chosen_children = False
            self.centre = None
            self.current_prod = []
        if name == "terminal":
            self.tree.addLeaf(self.current_terminal['id'], self.current_terminal)
            self.in_terminal=False

    def characters(self, content):
        if self.current_tag == "category" and content.strip():
            self.current_node.setCategory( content.strip() )
        if self.current_tag == "f" and content.strip():
            if self.current_arg == "tag" and self.in_terminal:
                self.current_terminal['tag'] = content.strip()
            else:
                self.current_node.addArgument(self.current_arg, content.strip())
        if self.current_tag == "orth" and self.in_terminal and content.strip():
            self.current_terminal["orth"] = content.strip()
        if self.current_tag == "base" and self.in_terminal and content.strip():
            self.current_terminal["base"] = content.strip()
            
    def endDocument(self):
        pass#self.tree.findHeads()

    """
    def count_probs(self, node_id):
        if node_id in self.nodes_probs:
            return self.nodes_probs[node_id]
        try:
            poss_prods = self.tree[node_id]
        except KeyError:
            poss_prods = []
        if not poss_prods:
            return 1.0
        for chosen, prod_dict in poss_p
            if prod_dict['productions']:
                prod_dict['prob'] = 1.0
                for prod in prod_dict['productions']:
                    try:
                        prod_dict['prob'] *= self.count_probs(prod) * self.count_probs(prod_dict['centre']) *\
                                             grammar[(self.nodes[node_id],\
                                                     self.nodes[prod])]
                    except KeyError:
                        prod_dict['prob'] = 0.0
            else:
                try:
                    prod_dict['prob'] = 1.0 *\
                                        self.count_probs(prod_dict['centre']) *\
                                        grammar[(self.nodes[node_id], self.nodes[prod_dict['centre']], None)]
                except KeyError:
                    prod_dict['prob'] = 0.0
        self.nodes_probs[node_id] = sorted(poss_prods, key=lambda x: x[1]['prob'])[0][1]['prob']
        return self.nodes_probs[node_id]



    def get_prob_tree(self, node_id):
        poss_prods = self.tree[node_id]
        if poss_prods:
            try:
                prod_dict = sorted(poss_prods, key=lambda x: x[1]['prob'])[0][1]
            except KeyError:
                prod_dict = poss_prods[0][1]
            self.prob_tree.append(( node_id, self.nodes_from_to[node_id] ))#self.prob_tree.append(( self.nodes[node_id], self.nodes_from_to[node_id] ))
            if prod_dict['centre']:
                self.get_prob_tree(prod_dict['centre'])
            for prod in prod_dict['productions']:
                self.prob_tree_dep.append((node_id, prod))
                self.get_prob_tree(prod)
            if self.nodes_from_to[node_id][1] - self.nodes_from_to[node_id][0] == 1:
                self.prob_tree_centres[node_id] = self.nodes_from_to[node_id][0]
            else:
                self.prob_tree_centres[node_id] = self.prob_tree_centres[prod_dict['centre']]
        else:
            if self.nodes_from_to[node_id][1] - self.nodes_from_to[node_id][0] == 1:
                self.prob_tree_centres[node_id] = self.nodes_from_to[node_id][0]

    def endDocument(self):
        print self.amb_count
        if u'0' in self.nodes:
            self.count_probs(u'0')
            self.get_chosen_tree(u'0')
            self.get_prob_tree(u'0')
            #nodes_to_expand = map(lambda x: x[0], self.prob_tree_dep)
            #while self.nodes_to_expand:
            #for ii in self.nodes_to_expand:
            #print self.nodes_to_expand, sorted(self.prob_tree_centres.keys(), key = lambda x: int(x))
            #    if ii[1] in self.prob_tree_centres:
            #        self.prob_tree_centres[ii[0]] = self.prob_tree_centres[ii[1]]
            #        self.nodes_to_expand.remove(ii)
            #print self.nodes_from_to
            self.precision = 1.0 * len(list((set(self.chosen_tree).intersection(set(self.prob_tree))))) / len(self.chosen_tree)
            self.recall = 1.0 * len(list((set(self.chosen_tree).intersection(set(self.prob_tree))))) / len(self.chosen_tree)
            #print self.precision, self.recall
            #print sorted(list(set(self.chosen_tree).difference(set(self.prob_tree))), key=lambda x: x[1])
            #print sorted(list(set(self.prob_tree).difference(set(self.chosen_tree))), key=lambda x: x[1])
            #print self.chosen_tree_centres

            self.chosen_tree_dep = map(lambda x: (self.chosen_tree_centres[x[1]], self.chosen_tree_centres[x[0]]), self.chosen_tree_dep)
            self.prob_tree_dep = map(lambda x: (self.prob_tree_centres[x[1]], self.prob_tree_centres[x[0]]), self.prob_tree_dep)
            for ii in range(len(self.chosen_tree_dep)+1):
                if ii not in map(lambda x: x[0], self.chosen_tree_dep):
                    self.chosen_tree_dep.append((ii, "root"))
            for ii in range(len(self.prob_tree_dep)+1):
                if ii not in map(lambda x: x[0], self.prob_tree_dep):
                    self.prob_tree_dep.append((ii, "root"))
                #print self.chosen_tree_dep, self.prob_tree_dep
            self.precision_dep = 1.0 * len(list(set(self.chosen_tree_dep).intersection( set(self.prob_tree_dep)))) / len(self.prob_tree_dep)
    """




#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

from graph_tool.centrality import pagerank
from wosedon.algorithms.wsdalgorithminterface import WSDAlgorithmInterface
from wosedon.ranking.wsd_ranking import WSDRanking

class GTPersonalizedPRNormIt(WSDAlgorithmInterface):
  """!
  Personalized Page Rank algorithm based on the set of synsets which
  correspond to lemmas from the analyzed context. Values in personalized
  vector "v" which corespond to all synsets which belong to lemmas from 
  the analyzed context depends on the number of lemmas meanings.

  If lemma has N meanings (synsets) then values in personalized vector 
  "v" for all its meanings (synsets) will be 1/N. If this same synset
  occur in two lemmas from the analyzed context then boths values are added.

  For example:
  \code
  Lemma    Set of synsets for lemma
  kot      1, 2, 3, 4, 5
  pies     5, 6, 7

  Personalized vector "v":
  Synset ID             Value
  1 (kot-1)             1/5
  2 (kot-2)             1/5
  3 (kot-3)             1/5
  4 (kot-4)             1/5
  5 (kot-5) & (pies-1)  1/5 + 1/3
  6 (pies-2)            1/3
  7 (pies-3)            1/3
  \endcode

  Remaining synsets which do not belong to lemmas from the analyzed context
  have value equal to 0 in personalized vector "v".

  Additionally after each iteration values which corespond to lemmas from the
  analyzed context in personalized vector "v" are normalized. Firstly all values
  from personalized vector "v", for only this synsets which belong to one lemma,
  are summed and the new value for this synsets are calculated as division old
  value over sum.

  For example:
  \code
  Synset ID             Old value    New value
  1 (kot-1)             848.29       0.20
  2 (kot-2)             12.12        0.003
  3 (kot-3)             2382.12      0.56
  4 (kot-4)             1000.22      0.24
  5 (kot-5) & (pies-1)  12.11        0.00002
  6 (pies-2)            0.12         0.001
  7 (pies-3)            123.11       0.99
  \endcode
  """

  def __init__(self, str_name = 'GTPersonalizedPRNormIt'):
    """!
    @param str_name - default value same as the class name "GTPersonalizedPRNormIt"
    """
    super(GTPersonalizedPRNormIt, self).__init__(str_name)
    self._context_node_dict = None
  
  def _normalize_gt_ranking(self, wsd_context, g, ranking, lemma_on_node_dict):
    """!
    Normalize Page Rank ranking.

    @param wsd_context - object of WSDContext class
    @param g - Graph object from graph_tool library
    @param ranking - PropertyMap (see: http://graph-tool.skewed.de/static/doc/graph_tool.html?highlight=new_vertex_property#graph_tool.Graph.new_vertex_property) of ranking values 
    @param lemma_on_node_dict - dictionary where key is lemma and value is
                                set of synsets (nodes from the graph) belong
                                to this lemma
    @return PropertyMap (see: http://graph-tool.skewed.de/static/doc/graph_tool.html?highlight=new_vertex_property#graph_tool.Graph.new_vertex_property) of new ranking values
    """
    l_pos = set([
      (wsd_context.get_token_lemma_str(t),
       wsd_context.get_token_coarse_pos(t)) for t in wsd_context.tokens()])
      
    for lemma, pos_str in l_pos:
      s = 0
      for vsyn in lemma_on_node_dict[(lemma, pos_str)]:
        s += ranking[vsyn]
      for vsyn in lemma_on_node_dict[(lemma, pos_str)]:
        ranking[vsyn] /= s
    return ranking

  def prepare_v(self, wsd_context, graph):
    """!
    Create personalized vector "v".

    @param wsd_context - object of WSDContext class
    @param graph - object of BaseGraph class
    @return PropertyMap (see: http://graph-tool.skewed.de/static/doc/graph_tool.html?highlight=new_vertex_property#graph_tool.Graph.new_vertex_property) of personalized vector "v"
    """
    v = graph.use_graph_tool().new_vertex_property("double")
    for node in self._context_node_dict:
      v[node.use_graph_tool()] = self._multiply_factor * self._context_node_dict[node]
    return v 
  
  def run(self, wsd_context, graph, options, resources):
    """!
    Disambiguate analyzed context.

    @param wsd_context - object of WSDContext class
    @param graph - object of BaseGraph class
    @param options - object of AlgorithmOptions class
    @param resources - object of Resources class
    @return tuple of object of WSDRanking class and number of algorithm iterations
    """
    wsd_rank = WSDRanking()
    (lemma_on_node_dict, lemma_on_only_synset_node_dict) = \
      wsd_rank.get_lemma_on_node_dict(wsd_context, graph, options.ini_nodes())
    self._set_context_node_dict(lemma_on_node_dict)

    pers_v = self.prepare_v(wsd_context, graph)
    ranking = None
    for i in range(0, options.max_iter()):
      (ranking, ret_iter) = pagerank(graph.use_graph_tool(), 
                                   pers = pers_v,
                                   prop = ranking,
                                   max_iter = 2,
                                   damping = options.damping_factor(),
                                   ret_iter = True,
                                   weight = graph.use_graph_tool().ep["weight"])
      ranking = self._normalize_gt_ranking(wsd_context, graph.use_graph_tool(), ranking, lemma_on_node_dict)
    
    ranking = graph.ungraph_tool(ranking)
    for (lemma, pos_str) in lemma_on_only_synset_node_dict.iterkeys():
      wsd_rank.set_ranking_for_lemma(lemma, pos_str, ranking)

    return (wsd_rank, ret_iter)

  def _set_context_node_dict(self, lemma_on_node_dict):
    """!
    Create a dictionary where key is object of node from the graph which
    corespond to the synset which belong to lemma from the analyzed context.
    Whereas value is a value which synset from the key will be have in
    personalized vector "v".

    @param lemma_on_node_dict - dictionary where key is (lemma, pos_str) and
                                value is set of synsets (nodes from the graph)
                                belong to this lemma
    """
    self._context_node_dict = {}
    for nodes in lemma_on_node_dict.itervalues():
      synset_set_size = float(len(nodes))
      for node in nodes:
        if self._context_node_dict.has_key(node):
          self._context_node_dict[node] += 1.0 / synset_set_size
        else:
          self._context_node_dict[node] = 1.0 / synset_set_size



# -*- encoding: utf-8 -*-
__author__ = 'nika'

from xml.sax import make_parser
from treeparser import TreeParser
from productionparseractpcfg import makeActPcfg
from node import Node
from tree import Tree
import glob
from multiprocessing import Pool
from sklearn.cross_validation import KFold


############################# CZYTANIE WSZYSTKICH PLIKÓW

all_files = []
for f in glob.iglob("/home/nika/zrobione140523/*/*/*-s.xml"):
    all_files.append(f)
print len(all_files)
all_files = all_files
############################# WALIDACJA KRZYŻOWA

def ev(args):
    f, grammar = args
    parser = make_parser()
    handler = TreeParser(f)
    parser.setContentHandler(handler)
    parser.parse(f)
    tree = handler.getTree()
    return tree.act_pcfg(grammar), tree.countChildrenZadnie()#[]#act_pcfg(grammar), tree.findMistakesSimple()

def count_ev(list_of_tuples):
    lll = []
    for ii in range(len(list_of_tuples[0])):
        lll.append(sum(map(lambda x: x[ii], list_of_tuples))/len(list_of_tuples))
    return lll

def count_ev_repr(list_of_tuples):
    lll = []
    for ii in range(len(list_of_tuples[0][0])):
        lll.append(sum(map(lambda x: x[0][ii], list_of_tuples))/len(list_of_tuples))
    for ii in range(len(list_of_tuples[0][1])):
        lll.append(sum(map(lambda x: x[1][ii], list_of_tuples))/len(list_of_tuples))
    return lll

# self.agreed_count, self.agreed_strict_count, self.chosen_nodes_count, self.disamb_nodes_count, self.dep_agreed_count, self.dep_count
def repr_stats(list_of_stats):
    all_sum = []
    for ii in range(len(list_of_stats[0])):
        all_sum.append(sum(map(lambda x:x[ii], list_of_stats)))
    result = [[1.0*all_sum[0]/all_sum[2], 1.0*all_sum[0]/all_sum[3], 1.0*all_sum[1]/all_sum[2], 1.0*all_sum[1]/all_sum[3], 1.0*all_sum[4]/all_sum[5]]]
    tmp = map(lambda x:[1.0*x[0]/x[2], 1.0*x[0]/x[3], 1.0*x[1]/x[2], 1.0*x[1]/x[3], 1.0*x[4]/x[5]], list_of_stats)
    all_sum = []
    for ii in range(len(tmp[0])):
        all_sum.append(sum(map(lambda x:x[ii], tmp))/len(tmp))
    result.append( all_sum )
    return result




all_lex_pcfg = []
mistakes =[]
for train, test in KFold(len(all_files), 10):
    print len(train), len(test)
    train_files = map(lambda x: all_files[x], train)
    print "making grammar ..."
    grammar = makeActPcfg(train_files, 20)
    test_files = map(lambda x: (all_files[x], grammar), test)
    print "evaluating ..."#, test_files
    pool = Pool(processes=20)              # start 4 worker processes
    #for ii in test_files: ev(ii)
    results = pool.map(ev, test_files)
    all_lex_pcfg.append(repr_stats(map(lambda x: x[0],results)))
    mistakes += map(lambda x:x[1], results)
    print "***************************************************************************************"
    print "RESULT", len(all_lex_pcfg), ":          ",  all_lex_pcfg[-1]


print "***************************************************************************************"
print "RESULT:          ",   count_ev_repr(all_lex_pcfg)

"""

mistakes_dict = {}
for ii in mistakes:
    for jj in ii:
        if jj in mistakes_dict:
            mistakes_dict[jj] +=1
        else:
            mistakes_dict[jj] =1

for jj in mistakes:
    for mistake in jj.keys():
        if mistake in mistakes_dict:
            mistakes_dict[mistake] += jj[mistake]
        else:
            mistakes_dict[mistake] = jj[mistake]


import pickle

for ii in sorted(mistakes_dict, key= lambda x:-mistakes_dict[x]):
    print mistakes_dict[ii], ii

pickle.dump(mistakes_dict, open("count-zdanie-dump.pkl", "w"))
"""



#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import itertools, logging

import graph_tool as gt
from graph_tool import Graph, load_graph
from graph_tool.generation import graph_union
from graph_tool.stats import remove_parallel_edges



class BaseNode(object):
  __slots__ = ['_graph', '_node']
  
  def __init__(self, graph, node):
    """
    @param graph:  graph to which the node belongs
    @type  graph:  graph_tool.Graph
    
    @param node:  underlying node
    @type  node:  graph_tool.Vertex
    """
    self._graph = graph
    self._node = node
  
  def use_graph_tool(self):
    """
    Returns underlying graph_tool.Vertex. It should be avoided at all costs. 
    """
    return self._node
  
  def all_edges(self):
    for e in self._node.all_edges():
      yield BaseEdge(self._graph, e)
  
  def all_neighbours(self):
    for n in self._node.all_neighbours():
      yield BaseNode(self._graph, n)
  
  def in_degree(self):
    return self._node.in_degree()
  
  def out_degree(self):
    return self._node.out_degree()
  
  def __getattr__(self, name):
    """
    allows acces to properties of graph
    """
    return self._graph.vp[name][self._node]
  
  
  def __setattr__(self, name, value):
    """
    allows acces to properties of graph
    """
    if name in self.__slots__:
      super(BaseNode, self).__setattr__(name, value)
    else:
      self._graph.vp[name][self._node] = value
  
  
  def __eq__(self, another):
    if isinstance(another, BaseNode):
      return self._node == another._node and self._graph == another._graph
    return self._node == another
    
  def __ne__(self, another):
    if isinstance(another, BaseNode):
      return self._node != another._node or self._graph != another._graph
    return self._node != another
  
  def __hash__(self):
    return hash(self._node)
  
  def __str__(self):
    return str(self._node)
  
  def __repr__(self):
    return repr(self._node)
  
  def __int__(self):
    return int(self._node)
  
  def copy(self):
    return BaseNode(self._graph, self._node)



class BaseEdge(object):
  __slots__ = ['_graph', '_edge']
  
  def __init__(self, graph, edge):
    """
    @param graph:  graph to which the edge belongs
    @type  graph:  graph_tool.Graph
    
    @param edge:  underlying edge
    @type  edge:  graph_tool.Edge
    """
    self._graph = graph
    self._edge = edge
  
  def use_graph_tool(self):
    """
    Returns underlying graph_tool.Edge. It should be avoided at all costs. 
    """
    return self._edge
  
  def __getattr__(self, name):
    """
    allows acces to properties of graph
    """
    return self._graph.ep[name][self._edge]
  
  
  def __setattr__(self, name, value):
    """
    allows acces to properties of graph
    """
    if name in self.__slots__:
      super(BaseEdge, self).__setattr__(name, value)
    else:
      self._graph.ep[name][self._edge] = value
  
  
  def __eq__(self, another):
    return self._edge == another._edge and self._graph == another._graph
    
  def __ne__(self, another):
    return self._edge != another._edge or self._graph != another._graph
  
  def __hash__(self):
    return hash(self._edge)
  
  def __str__(self):
    return str(self._edge)
  
  def __repr__(self):
    return repr(self._edge)
  
  def copy(self):
    return BaseEdge(self._graph, self._edge)
  
  
  def source(self):
    return BaseNode(self._graph, self._edge.source())
    
  def target(self):
    return BaseNode(self._graph, self._edge.target())





class BaseGraph(object):
  """
  Class representing a graph. We do not use pure graph_tool.Graph for we want
  to be able to easily change this library. Neither we use inheritance
  as graph_tool has inconvenient licence.
  """
  
  def __init__(self):
    self._g = None
    self._node_dict = {}
    self._syn_to_vertex_map = None
  
  def use_graph_tool(self):
    """
    Returns underlying graph_tool.Graph. It should be avoided at all costs. 
    """
    return self._g
  
  def get_node_for_synset_id(self, syn_id):
    """
    Lazy function to makes the map of synset identifiers to nodes into 
    the graph. The building of map is made only on the first funcion call.
    The first and the next calls of this function will return the built map.
    """
    if not self._syn_to_vertex_map:
      self._syn_to_vertex_map = {}
      for node in self.all_nodes():
        if node.synset:
          synset_id = node.synset.synset_id
          self._syn_to_vertex_map[synset_id] = node
    return self._syn_to_vertex_map.get(syn_id, None)

  def pickle(self, filename):
    self._g.save(filename)

  def unpickle(self, filename):
    self._g = load_graph(filename)

  def init_graph(self, drctd = False):
    self._g = Graph(directed = drctd)

  def copy_graph_from(self, g):
    self._g = g._g.copy()

  def set_directed(self, drctd):
    self._g.set_directed(drctd)

  def is_directed(self):
    return self._g.is_directed()

  def merge_graphs(self, g1, g2):
    self._g = graph_union(g1._g, g2._g, internal_props = True)

  # Node operations:
  def all_nodes(self):
    for node in self._g.vertices():
      yield BaseNode(self._g, node)

  def create_node_attribute(self, name, kind, value=None):
    if name not in self._g.vertex_properties:
      node_attr = self._g.new_vertex_property(kind, value)
      self._g.vertex_properties[name] = node_attr

  def create_node_attributes(self, node_attributes_list):
    for attr in node_attributes_list:
      if attr[0] not in self._g.vertex_properties:
        node_attr = self._g.new_vertex_property(attr[1])
        self._g.vertex_properties[attr[0]] = node_attr

  def add_node(self, name, node_attributes_list=[]):
    if name not in self._node_dict:
      new_node = self._g.add_vertex()
      self._node_dict[name] = BaseNode(self._g, new_node)
      for attr in node_attributes_list:
        self._g.vertex_properties[attr[0]][new_node] = attr[1]
    return self._node_dict[name]

  def get_node(self, name):
    return self._node_dict[name]
  
  def remove_node(self, name):
    self._g.remove_vertex(self._node_dict[name]._node)
    del self._node_dict[name]

  def nodes_filter(self, nodes_to_filter_set, inverted = False):
    node_filter = self._g.new_vertex_property("bool")
    for n in self.all_nodes():
      if n in nodes_to_filter_set:
        node_filter[n._node] = False
      else:
        node_filter[n._node] = True
    self._g.set_vertex_filter(node_filter, inverted)
    self._g.purge_vertices()

  # Edge operations:
  def num_edges(self):
    return self._g.num_edges()
  
  def all_edges(self):
    for e in self._g.edges():
      yield BaseEdge(self._g, e)
  
  def get_edges_between(self, source, target):
    """
    Return all edges between source and target. Source and target can be either
    BaseNode or integer.
    """
    if isinstance(source, BaseNode):
      source = source._node
    if isinstance(target, BaseNode):
      target = target._node
    for e in self._g.edge(source, target, all_edges=True):
      yield BaseEdge(self._g, e)
    
  def get_edge(self, source, target, add_missing=False):
    """
    Return some edge between source and target. Source and target can be either
    BaseNode or integer.
    """
    if isinstance(source, BaseNode):
      source = source._node
    if isinstance(target, BaseNode):
      target = target._node
    e = self._g.edge(source, target, add_missing)
    if e is not None:
      return BaseEdge(self._g, e)
    else:
      return None

  def create_edge_attribute(self, name, kind, value=None):
    if name not in self._g.edge_properties:
      edge_attr = self._g.new_edge_property(kind, value)
      self._g.edge_properties[name] = edge_attr

  def alias_edge_attribute(self, name, alias): 
    self._g.edge_properties[alias] = self._g.edge_properties[name]

  def create_edge_attributes(self, edge_attributes_list): 
    for attr in edge_attributes_list:
      if attr[0] not in self._g.edge_properties:
        edge_attr = self._g.new_edge_property(attr[1])
        self._g.edge_properties[attr[0]] = edge_attr

  def add_edge(self, parent, child, edge_attributes_list = [], simply = False):
    if simply:
      new_edge = self._g.add_edge(parent._node, child._node)
    else:
      if self._node_dict.has_key(parent) \
      and self._node_dict.has_key(child):
        n_parent = self._node_dict[parent]._node
        n_child = self._node_dict[child]._node
        new_edge = self._g.add_edge(n_parent, n_child)
    for attr in edge_attributes_list:
      self._g.edge_properties[attr[0]][new_edge] = attr[1]
    return BaseEdge(self._g, new_edge)

  def edges_filter(self, edges_to_filter_set):
    edge_filter = self._g.new_edge_property("bool")
    for e in self.all_edges():
      if e in edges_to_filter_set:
        edge_filter[e._edge] = False
      else:
        edge_filter[e._edge] = True
    self._g.set_edge_filter(edge_filter)
    self._g.purge_edges()

  def remove_edge_duplicates(self):
    # WARNING 
    # Uwaga! Usuwa wszystkie krawedzie. Nie patrzy na typ krawedzi
    # ani na wage krawedzi, usuwa wszystko i zostawia tylko jedna
    # krawedz. Aby zostawic okreslone krawedzie nalezy repimplementowac
    # ta metode.
    remove_parallel_edges(self._g)
  
  def ungraph_tool(self, thingy):
    """
    Converts given data structure so that it no longer have any graph_tool dependencies.
    """
    logger = logging.getLogger(__name__)
    
    if type(thingy) == dict:
      return {self.ungraph_tool(k) : self.ungraph_tool(thingy[k]) for k in thingy}
    
    if type(thingy) == gt.PropertyMap:
      dct = {}
      if thingy.key_type() == 'v':
        for node in self.all_nodes():
          dct[node] = thingy[node.use_graph_tool()]
      elif thingy.key_type() == 'e':
        for edge in self.all_edges():
          dct[edge] = thingy[edge.use_graph_tool()]
      else:
        logger.error('Unknown property type %s', thingy.key_type())
        raise NotImplemented
      return dct
    
    if type(thingy) == set:
      return {self.ungraph_tool(k) for k in thingy}
    
    if type(thingy) == list:
      return [self.ungraph_tool(k) for k in thingy]
    
    if type(thingy) == gt.Node:
      return BaseNode(self, thingy)
    
    if type(thingy) == gt.Edge:
      return BaseEdge(self, thingy)
    
    return thingy



















# -*- coding: utf-8 -*-

import unittest
from wosedon.utils.resources import Resources
from wosedon.utils.buildoptions import BuildOptions
from wosedon.gbuilders.cclgraphbuilder import CCLGraphBuilder





def leniter(i):
  """ Lenght of any iterable """
  return sum(1 for e in i)


class TestCCLGB_DirectnessAndDuplicates(unittest.TestCase):
  
  def setUp(self):
    self.res = Resources()
    self.res.set_ccl_graph_file('tests/#Data/cclgraph.xml')
    self.opts = BuildOptions()
  
  
  def test_undirected_not_unique(self):
    self.opts.set_directed_graph(False)
    self.opts.set_unique_edges(False)
    
    g = CCLGraphBuilder(self.res, self.opts).build_graph()
    
    self.assertEquals(leniter(g.get_edges_between(3, 6)), 2)
    self.assertEquals(leniter(g.get_edges_between(5, 8)), 2)
    self.assertEquals(leniter(g.get_edges_between(0, 10)), 2)
    self.assertEquals(leniter(g.get_edges_between(5, 1)), 1)
    self.assertEquals(leniter(g.get_edges_between(1, 5)), 1)
  
  
  def test_directed_not_unique(self):
    self.opts.set_directed_graph(True)
    self.opts.set_unique_edges(False)
    
    g = CCLGraphBuilder(self.res, self.opts).build_graph()
    
    self.assertEquals(leniter(g.get_edges_between(3, 6)), 2)
    self.assertEquals(leniter(g.get_edges_between(8, 5)), 2)
    self.assertEquals(leniter(g.get_edges_between(0, 10)), 1)
    self.assertEquals(leniter(g.get_edges_between(5, 1)), 1)
    self.assertEquals(leniter(g.get_edges_between(1, 5)), 0)
    self.assertEquals(leniter(g.get_edges_between(7, 7)), 1)
  
  
  def test_undirected_unique(self):
    self.opts.set_directed_graph(False)
    self.opts.set_unique_edges(True)
    
    g = CCLGraphBuilder(self.res, self.opts).build_graph()
    
    for s in g.all_nodes():
      for t in g.all_nodes():
        if s != t:
          self.assertLessEqual(leniter(g.get_edges_between(s, t)), 1)
  
  
  def test_directed_unique(self):
    self.opts.set_directed_graph(True)
    self.opts.set_unique_edges(True)
    
    g = CCLGraphBuilder(self.res, self.opts).build_graph()
    
    for s in g.all_nodes():
      for t in g.all_nodes():
        self.assertLessEqual(leniter(g.get_edges_between(s, t)), 1)
        



class TestCCLGB(unittest.TestCase):
  
  def setUp(self):
    self.res = Resources()
    self.res.set_ccl_graph_file('tests/#Data/cclgraph.xml')
    self.opts = BuildOptions()
    self.opts.set_directed_graph(True)
    self.opts.set_unique_edges(False)
  
  
  def test_edge_names(self):
    g = CCLGraphBuilder(self.res, self.opts).build_graph()
    
    for e in g.all_edges():
      self.assertIn(e.rel, ['in_sentence', 'in_paragraph', 'in_document'])






























from wosedon.lesk_functions.dictionarybased import DictionaryBased



class Intersection(DictionaryBased):
  """! How much definition intersects with context. """
  
  def __init__(self, resources, filter_obj, graph, options, str_name='Intersection'):
    super(Intersection, self).__init__(resources, filter_obj, graph, options, str_name)
  
  def compare(self, definition_data, context_data):
    coverage = 0
    for (lemma, pos_str) in definition_data:
      if (lemma, pos_str) in context_data:
        coverage += 1
    return float(coverage)
























#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import sys
from collections import defaultdict

from wosedon.basegraph import BaseGraph
from mergerinterface import MergerInterface

class SynsetsLUMerger(MergerInterface):
  """!
  Class for merging synsets graph and lexical units graph.

  The purpose of this class is to merge two graphs, namely synsets graph and 
  lexical units graph. Nodes from lexical units graph are connected to proper 
  nodes from synsets graph, which have them in their lexical units sets.
  """
  def __init__(self, resources, options, str_name = 'SynsetsLUMerger'):
    """!
    @param resources - object of Resources class contain resources from config 
                       file
    @param options   - object of MergeOptions class contain merge options from 
                       config file
    @param str_name  - default value same as the class name "SynsetsLUMerger"
    """
    super(SynsetsLUMerger, self).__init__(resources, options, str_name)

  def merge(self, g1, g2):
    """!
    Merge two given graphs, namely synsets graph and lexical units graph. The final 
    graph contain two types of nodes. Each synset node has an attribute named "synset", 
    to which is assigned synset object. Synset object consist of: synset_id and lu_set, 
    which contains lexical units objects. Whereas each lexical unit node has an 
    attribute named "lu", to which is assigned lexical unit object. Lexical unit 
    object consist of: lu_id, lemma, pos, variant and domain. Synset relation exist 
    between synsets nodes only. Lexical unit relation exist between lexical units nodes 
    only. Whereas new relations are added between synsets nodes and lexical units 
    nodes. If lu_id of lexical unit node occur in the object from lu_set of synset 
    node then new relation is added between them.
    
    The directness of the graph and uniqueness of edges are inherited from synsets graph.

    Warning: If the final graph is a directed graph then two edges will be added between 
    a synset node and a lexical unit node. First edge will be add from synset node to 
    lexical unit node and second from lexical unit node to synset node.

    @param g1 - synsets graph
    @param g2 - lexical units graph
    @return object of BaseGraph class
    """
    g = BaseGraph()
    g.init_graph(drctd = g1.is_directed())

    g.merge_graphs(g1, g2)

    lu_on_vertex_dict = defaultdict(set)
    for node in g.all_nodes():
      if node.synset:
        lu_set = node.synset.lu_set
        for lu in lu_set:
          lu_on_vertex_dict[lu.lu_id].add(node)

    for node in g.all_nodes():
      if node.lu:
        lu_id = node.lu.lu_id
        if lu_on_vertex_dict.has_key(lu_id):
          for synset_id_node in lu_on_vertex_dict[lu_id]:
            g.add_edge(node, synset_id_node, [("rel", 'syn-lu')], simply = True)
            if g.is_directed():
              g.add_edge(synset_id_node, node, [("rel", 'syn-lu')], simply = True)
        else:
          logging.getLogger(__name__).warning("No synset in the graph contains lemma of ID %d.", lu_id)

    return g



import unittest
from wosedon.utils.algorithmoptions import AlgorithmOptions

from wosedon.utils.resources import Resources
from wosedon.basegraph import BaseGraph
from wosedon.lesk_functions.leskfunctioninterface import LeskFunctionInterface
from wosedon.lesk_filters.leskfilterinterface import LeskFilterInterface

from wosedon.lesk_functions import *
from wosedon.lesk_filters import *


class TestAlgorithmOptions(unittest.TestCase):
  
  def setUp(self):
    self.opts = AlgorithmOptions()
    self.res = Resources()
    self.res.set_tagset('nkjp')
    self.g = BaseGraph()
  
  
  def test_type(self):
    self.opts.set_lesk_function('Cosine')
    self.assertIsInstance(self.opts.lesk_function(self.res, self.g), LeskFunctionInterface)
    self.opts.set_lesk_function('ExampleFunction')
    self.assertIsInstance(self.opts.lesk_function(self.res, self.g), LeskFunctionInterface)
    self.opts.set_lesk_function('Intersection')
    self.assertIsInstance(self.opts.lesk_function(self.res, self.g), LeskFunctionInterface)
    
    self.opts.set_lesk_filter('POS')
    self.assertIsInstance(self.opts.lesk_filter(None), LeskFilterInterface)
    self.opts.set_lesk_filter('Yes')
    self.assertIsInstance(self.opts.lesk_filter(None), LeskFilterInterface)
    self.opts.set_lesk_filter('WordsPOS')
    self.assertIsInstance(self.opts.lesk_filter(None), LeskFilterInterface)
  
  
  def test_default(self):
    self.assertIsInstance(self.opts.lesk_filter(None), Yes)




#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#
from graph_tool.all import *
from graph_tool.draw import graph_draw
from graph_tool.topology import shortest_path

import matplotlib.pyplot
from collections import defaultdict

class Visualisation():
  def __init__(self):
    pass

  def make_visualisation(self, wsd_context, graph, wsd_rank, v_dir):
    (lemma_on_node_dict, lemma_on_only_synset_node_dict) = \
      wsd_rank.get_lemma_on_node_dict(wsd_context, graph)

    nodes_from_context_list = set()
    for node_set in lemma_on_only_synset_node_dict.itervalues():
      nodes_from_context_list = nodes_from_context_list.union(node_set)
    nodes_from_context_list = list(nodes_from_context_list)

    for t in wsd_context.tokens():
      lemma = wsd_context.get_token_lemma_str(t)
      pos_str = wsd_context.get_token_coarse_pos(t)
      ranking = wsd_rank.get_ranking_for_lemma(lemma, pos_str)
      if not ranking:
        continue

      v_graph = graph.G().copy()

      # PageRank dla wierzcholka
      node_attr = v_graph.new_vertex_property("double")
      v_graph.vp["PageRank"] = node_attr
      # PropertyMap do oznaczenia czy dany synset jest
      # synsetem z kontekstu
      node_attr = v_graph.new_vertex_property("bool")
      v_graph.vp["context_synset"] = node_attr
      # Etykieta wierzcholka
      node_attr = v_graph.new_vertex_property("string")
      v_graph.vp["vertex_text"] = node_attr
      # Kolor etykiety wierzcholka
      node_attr = v_graph.new_vertex_property("string")
      v_graph.vp["vertex_text_color"] = node_attr
      # Kolor wypelnienia wierzcholka
      node_attr = v_graph.new_vertex_property("vector<double>")
      v_graph.vp["vertex_fill_color"] = node_attr
      # Pozycje wezlow
      node_attr = v_graph.new_vertex_property("vector<double>")
      v_graph.vp["pos"] = node_attr

      nodes_not_to_filter_set = set()
      for i in range(len(nodes_from_context_list)-1):
        source = nodes_from_context_list[i]
        #v_graph.vp["context_synset"][source] = True
        for j in range(1, len(nodes_from_context_list)):
          target = nodes_from_context_list[j]
          #v_graph.vp["context_synset"][target] = True
          nlist, elist = shortest_path(v_graph, source, target)
          for n in nlist:
            nodes_not_to_filter_set.add(n)
            v_graph.vp["PageRank"][n] = ranking[n]

      sorted_pagerank_set = sorted(set(v_graph.vp["PageRank"].a))
      color_for_ranking_dict = {}
      for i, rank in enumerate(sorted_pagerank_set):
        color_for_ranking_dict[rank] = float(i) / float((len(sorted_pagerank_set) - 1))

      for node in lemma_on_only_synset_node_dict[(lemma, pos_str)]:
        v_graph.vp["context_synset"][node] = True

      # Odfiltrowanie wierzcholkow, ktore nie znalazly sie
      # na sciezkach pomiedzy wierzcholkami z kontekstu
      node_filter = v_graph.new_vertex_property("bool")
      for n in v_graph.vertices():
        if n in nodes_not_to_filter_set:
          node_filter[n] = True
        else:
          node_filter[n] = False
      v_graph.set_vertex_filter(node_filter)
      v_graph.purge_vertices()

      nouns_nodes_not_to_filter_set = set()
      verbs_nodes_not_to_filter_set = set()
      for n in v_graph.vertices():
        for lu in v_graph.vp["synset"][n].lu_set:
          # Dodanie wezla grafu do odpowiedniego dla
          # czesci mowy zbioru
          if lu.pos == 2:
            nouns_nodes_not_to_filter_set.add(n)
          elif lu.pos == 1:
            verbs_nodes_not_to_filter_set.add(n)
          # Ustawienie etykiety dla kazdego wezla
          vertex_text = lu.lemma + '-' + str(lu.variant)
          v_graph.vp["vertex_text"][n] = vertex_text
          # Ustawienie koloru etykiety
          # Etykieta jest kolorowana jezeli synset nalezy do kontekstu
          if v_graph.vp["context_synset"][n]:
            v_graph.vp["vertex_text_color"][n] = "red"
          else:
            v_graph.vp["vertex_text_color"][n] = "black"

          v_graph.vp["vertex_fill_color"][n] = [color_for_ranking_dict[v_graph.vp["PageRank"][n]], 0, 0, 1]

          # Sprawdzana jest tylko pierwsza
          # jednoskta leksykalna w synsecie i
          # z niej pobierane sa wszystkie informacje o nim
          break

      # Utworzenie PropertyMap, w ktorej przechowywane sa rozmiary wierzcholkow
      vertex_size_vp = prop_to_size(v_graph.vp["PageRank"], mi=5, ma=15)

      # Ustalenie pozycji wierzcholkow
      max_neighbours = 0
      chosen_vertex = 0
      for v in v_graph.vertices():
        number_of_neighbours = len([child for child in v.out_neighbours()])
        if max_neighbours < number_of_neighbours:
          max_neighbours = number_of_neighbours
          chosen_vertex = v

      pos_dict = defaultdict(set)

      queue = []
      discovered_list = []
      discovered_list.append(chosen_vertex)
      v_graph.vp["pos"][chosen_vertex] = [0.0, 0.0]
      for ngb in chosen_vertex.out_neighbours():
        queue.append(ngb)

      while queue:
        v = queue.pop(0)
        if not v in discovered_list:
          for parent in v.out_neighbours():
            if parent in discovered_list:
              break
          number_of_exist_parent_child = 0
          number_of_parent_childs = 0
          for child in parent.out_neighbours():
            number_of_parent_childs += 1
            if child in discovered_list:
              number_of_exist_parent_child += 1
          side_size = x = y = 0.0
          side_size = ((float(number_of_parent_childs) - 1.0) * 500.0 + float(number_of_parent_childs) * vertex_size_vp[v]) / 2.0 - vertex_size_vp[v] / 2.0
          x = (v_graph.vp["pos"][parent][0] - float(side_size)) + (float(number_of_exist_parent_child) - 1.0) * 500.0 + float(number_of_exist_parent_child) * vertex_size_vp[v]
          y = v_graph.vp["pos"][parent][1] - 1000.0
          if pos_dict.has_key(y):
            for x_cord in sorted(pos_dict[y]):
              while x_cord - 200.0 < x and x < x_cord + 200.0:
                x += 100
          pos_dict[y].add(x)
          v_graph.vp["pos"][v] = [x, y]
          discovered_list.append(v)
          for ngb in v.out_neighbours():
            queue.append(ngb)


      # Rysowanie grafu dla wszystkich czesci mowy
      graph_draw(v_graph, 
                 pos=v_graph.vp["pos"],
                 vertex_fill_color=v_graph.vp["vertex_fill_color"],
                 vertex_size=prop_to_size(v_graph.vp["PageRank"], mi=5, ma=15),
                 vertex_text=v_graph.vp["vertex_text"], 
                 vertex_text_color=v_graph.vp["vertex_text_color"],
                 vertex_font_size=4,
                 vertex_text_position=1,
                 vertex_text_rotation=3.14 / 2,
                 output=v_dir + 'all_POS_' + lemma + '(' + pos_str + ').pdf')

      # Rysowanie grafu dla rzeczownikow
      for n in v_graph.vertices():
        if n in nouns_nodes_not_to_filter_set:
          node_filter[n] = True
        else:
          node_filter[n] = False
      v_graph.set_vertex_filter(node_filter)
      graph_draw(v_graph, 
                 pos=v_graph.vp["pos"],
                 vertex_fill_color=v_graph.vp["vertex_fill_color"],
                 vertex_size=prop_to_size(v_graph.vp["PageRank"], mi=5, ma=15),
                 vertex_text=v_graph.vp["vertex_text"], 
                 vertex_text_color=v_graph.vp["vertex_text_color"],
                 vertex_font_size=4,
                 vertex_text_position=1,
                 vertex_text_rotation=3.14 / 2,
                 output=v_dir + 'nouns_' + lemma + '(' + pos_str + ').pdf')

      # Rysowanie grafu dla czasownikow
      v_graph.clear_filters()
      for n in v_graph.vertices():
        if n in verbs_nodes_not_to_filter_set:
          node_filter[n] = True
        else:
          node_filter[n] = False
      v_graph.set_vertex_filter(node_filter)
      graph_draw(v_graph, 
                 pos=v_graph.vp["pos"],
                 vertex_fill_color=v_graph.vp["vertex_fill_color"],
                 vertex_size=prop_to_size(v_graph.vp["PageRank"], mi=5, ma=15),
                 vertex_text=v_graph.vp["vertex_text"], 
                 vertex_text_color=v_graph.vp["vertex_text_color"],
                 vertex_font_size=4,
                 vertex_text_position=1,
                 vertex_text_rotation=3.14 / 2,
                 output=v_dir + 'verbs_' + lemma + '(' + pos_str + ').pdf')


import corpus2
import codecs, os, sys, argparse

import domains as dom
import context.context as ccc
from npsemrel.carrot.db import db
from collections import defaultdict

def make_parser():
  desc = 'Narzedzie do porownywania wynikow WSD dla roznych modeli.'
  parser = argparse.ArgumentParser(description = desc)
  parser.add_argument('-o', '--out-file', dest = 'out_file', required = True)
  parser.add_argument('-i', '--input-files', nargs = '*', action = 'append', 
                      dest = 'in_files', required = True)
  parser.add_argument('-d', '--db-config', dest = 'db_config', required = True)
  parser.add_argument('-t', '--tagset', dest = 'tagset', default = 'nkjp')
  parser.add_argument('-e', '--only-incorrect', dest = 'only_incorrect', 
                      action = 'store_true')
  return parser

def get_models_comparsion_data(models_to_compare, tagset, ann_map):
  full_pos_mask = corpus2.get_attribute_mask(tagset, '')
  ctx  = ccc.WSDContext(tagset, [])

  statistic_dict = defaultdict(set)
  models_list = []

  for model_number, model in enumerate(models_to_compare):
    models_list.append((model_number, os.path.dirname(model)))
    with open(model, 'rt') as corpidx:
      for corpfile in corpidx:
        corpfile_full_path = os.path.join(os.path.dirname(model), corpfile).strip()
        print >> sys.stderr, 'Reading:', corpfile_full_path
        try:
          cclreader = corpus2.CclRelReader(tagset, corpfile_full_path, corpfile_full_path)
          document = cclreader.read()
          for paragraph in document.paragraphs():
            for sentence in paragraph.sentences():
              for token in sentence.tokens():
                md = token.get_metadata()
                if md:
                  ukb_syn_id = md.get_attribute('sense:ukb:syns_id')
                  ukb_syn_unitsstr = md.get_attribute('sense:ukb:unitsstr')
                  ukb_syn_rank = md.get_attribute('sense:ukb:syns_rank')

                  if not ukb_syn_id:
                    continue

                  attribs = md.attributes()
                  for attr_k, attr_v in attribs.iteritems():
                    if attr_k.startswith('sense:wsd_'):
                      if not ann_map.has_key(attr_v):
                        print >> sys.stderr, 'Brak klucza %s w slowniku!' % (attr_v)
                      else:
                        # Lemat
                        lemma = str(token.get_preferred_lexeme(tagset).lemma())
                        # POS
                        tag = token.get_preferred_lexeme(tagset).tag()
                        pos_mask = tag.get_masked(full_pos_mask)
                        pos_str = ctx.convert_to_coarse_pos(
                          tagset.tag_to_symbol_string(pos_mask))

                        key = (corpfile,
                               sentence.id(),
                               lemma,
                               pos_str,
                               attr_v,
                               ', '.join(str(syn_id) for syn_id in ann_map[attr_v].keys()))

                        value = (model_number,
                                 ukb_syn_unitsstr,
                                 ukb_syn_id,
                                 ukb_syn_rank)

                        statistic_dict[key].add(value)
        except Exception, e:
          print >> sys.stderr, 'Error: ', e

  return (statistic_dict, models_list)

def models_comparsion(statistic_dict, models_list, out_file):
  with open(out_file, 'wt') as ofile:
    set_column_name(ofile, models_list)

    for key, value_set in statistic_dict.iteritems():
      union = set()
      for value in value_set:
        union.add(value[2])
      if len(union) > 1:
        set_data(ofile, key, value_set)

def models_comparsion_only_incorrect(statistic_dict, models_list, out_file):
  with open(out_file, 'wt') as ofile:
    set_column_name(ofile, models_list)

    for key, value_set in statistic_dict.iteritems():
      correct_syn_id_list = [int(syn_id) for syn_id in key[5].split(', ')]
      for value in value_set:
        if not int(value[2]) in correct_syn_id_list:
          set_data(ofile, key, value_set)
          break

def set_column_name(ofile, models_list):
  models_names = ';;;'.join([model_pth for (model_number, model_pth) in models_list])
  models_data = ';'.join(['WSD synset string;WSD synset ID;Ranking' for i in range(len(models_list))])

  ofile.write('%s;%s;%s;%s;%s;%s;%s\n;;;;;;%s\n' % (
              'Document',
              'Sentence ID',
              'Lemma',
              'POS',
              'Manual synset ID',
              'Manual synset string',
              models_names,
              models_data))

def set_data(ofile, key, value_set):
  sort_value_list = sorted(list(value_set), key = lambda model: model[0])
  models_data = ';'.join(['%s;%s;%s' % (value[1], value[2], value[3]) if value_number == value[0] else ';;' for value_number, value in enumerate(sort_value_list)])

  ofile.write('%s;%s;%s;%s;%s;%s;%s\n' % (
              key[0].strip(),
              key[1],
              key[2],
              key[3],
              key[5],
              key[4],
              models_data))

def make_key(comment):
  return comment.replace('WSD', '').replace(' ', '').replace('#', '-').encode('utf-8')

def make_anotation_dictionary(dbconnection):
  ann_dict = {}

  print >> sys.stderr, 'Getting WSD comments from DB...',
  query = "SELECT SYN.comment, SYN.id, LU.lemma, LU.variant, LU.domain, LU.pos " \
          "FROM lexicalunit LU " \
          "JOIN unitandsynset UAS on (LU.ID = UAS.LEX_ID) " \
          "JOIN synset SYN on (SYN.id = UAS.SYN_ID) " \
          "WHERE SYN.comment LIKE '%wsd%';"
  cursor = dbconnection.cursor()
  cursor.execute(query)

  for row in cursor.fetchall():
    comments = row[0].split(';')[0].split(',')
    synset_id = int(row[1])

    lu_lemma = str(row[2].encode('utf-8')).replace(' ', '_')
    lu_pos = int(row[5])
    lu_variant = int(row[3])
    lu_domain = int(row[4])
    lu_domain_str = dom.DOMAINS[lu_domain] if dom.DOMAINS.has_key(lu_domain) else str(lu_domain)
    manual_synset_str = ('%s.%d(%d:%s) ' % \
                        (lu_lemma, 
                         lu_variant, 
                         lu_domain, 
                         lu_domain_str))

    for comment in comments:
      comment = make_key(comment)
      if ann_dict.has_key(comment):
        if ann_dict[comment].has_key(synset_id):
          ann_dict[comment][synset_id][0] += ' ' + manual_synset_str
          if lu_pos != ann_dict[comment][synset_id][1]:
            print >> sys.stderr, 'Dla synsetu', synset_id, 'wystapily rozne POSy!'
        else:
          ann_dict[comment][synset_id] = [manual_synset_str, lu_pos]
      else:
        ann_dict[comment] = defaultdict(list)
        ann_dict[comment][synset_id] = [manual_synset_str, lu_pos]
  print >> sys.stderr, ' Done!'

  return ann_dict

def main(argv = None):
  parser = make_parser()
  args = parser.parse_args(argv)
  tagset = corpus2.get_named_tagset(args.tagset)

  print >> sys.stderr, 'Connecting to DB...',
  dbcon = db.DB()
  dbconnection = dbcon.connect(args.db_config)
  if not dbconnection:
    print >> sys.stderr, 'Cannot connect to DB!'
    exit(1)
  print >> sys.stderr, ' Done!'

  models_to_compare = []
  for models_list in args.in_files:
    for model in models_list:
      models_to_compare.append(model)

  annotations = make_anotation_dictionary(dbconnection)

  statistic_dict = defaultdict(set)
  models_list = []
  (statistic_dict, models_list) = get_models_comparsion_data(models_to_compare, tagset, annotations)

  if args.only_incorrect:
    models_comparsion_only_incorrect(statistic_dict, models_list, args.out_file)
  else:
    models_comparsion(statistic_dict, models_list, args.out_file)

if __name__ == '__main__':
  main()


from wosedon.algorithms.wsdalgorithminterface import WSDAlgorithmInterface
from wosedon.ranking.wsd_ranking import WSDRanking

class LeskAlg(WSDAlgorithmInterface):
  """!
  The Lesk algorithm is based on the assumption that words in a given
  "neighborhood" (section of text) will tend to share a common topic.
  This version of the algorithm allows to use chosen functions for comparison
  between definition of ambiguous word and its neighborhood.
  """

  def __init__(self, str_name = 'LeskAlg'):
    super(LeskAlg, self).__init__(str_name)
    self.node_data = {}  # {synset_id : definition data returned by function}

  def prepare_v(self, wsd_context, graph):
    """! Required by WSDAlgorithmInterface. """
    pass

  def run(self, wsd_context, graph, options, resources):
    """!
    Disambiguate analyzed context.

    @type wsd_context: WSDContext
    @type graph: BaseGraph
    @type options: AlgorithmOptions
    @type resources: Resources
    
    @rtype (WSDRanking, int)
    @return: tuple containing WSDRanking and number of algorithm iterations
    """
    
    function = options.lesk_function(resources, graph)
    context_data = function.prepare_context(wsd_context)
    
    wsd_rank = WSDRanking()
    (lemma_on_node_dict, lemma_on_only_synset_node_dict) = \
      wsd_rank.get_lemma_on_node_dict(wsd_context, graph, options.ini_nodes())
    
    func_cache = {}
    
    for (lemma, pos_str), nodes in lemma_on_node_dict.iteritems():
      for node in nodes:
        # TODO It assumes that there exists exactly one graph during whole program, which is true for now.
        if node not in self.node_data:
          self.node_data[node] = function.prepare_node(node)
        
        if node not in func_cache:
          func_cache[node] = function.compare(
              self.node_data[node],
              context_data
          )
      
      wsd_rank.set_ranking_for_lemma(lemma, pos_str, func_cache, create_new = False)
    
    return (wsd_rank, 0)











# -*-  encoding: utf-8 -*-
__author__ = 'nika'

from xml.sax import make_parser
from treeparser import TreeParser
import os
import pickle
import sys
import xml.dom.minidom
import codecs

try:
    f = sys.argv[1]
except IndexError:
    print("ERROR: no filename given")
    exit()

# parse xml file for future processing
dom = xml.dom.minidom.parse(sys.argv[1])
nodes = dom.getElementsByTagName("node")

if nodes: # if there is anything to disambiguate
    # load grammar
#    grammar = pickle.load(open("grammars/pcfg-tfw-130718.pkl"))
    path = os.path.join(os.path.dirname(__file__), "grammars/pcfg-tfw-150326.pkl")
    grammar = pickle.load(open(path))
    # make parser, parse tree
    parser = make_parser()
    handler = TreeParser(f)
    parser.setContentHandler(handler)
    parser.parse(f)
    tree = handler.getTree()

    # disambiguation
    tree.act_pcfg(grammar)
    disamb_nodes = tree.getDisambNodes()

# update chosen nodes (if any)
for node in nodes:
    if node.attributes["nid"].value in disamb_nodes:
        node.attributes["chosen"] = "true"
        children_all = node.getElementsByTagName("children")
        for children in children_all:
            chosen = True
            for child in children.getElementsByTagName("child"):
                if child.attributes["nid"].value not in disamb_nodes:
                    chosen = False
            if chosen:
                children.attributes["chosen"] = "true"
    else:
        node.attributes["chosen"] = "false"

new_f = sys.argv[1].rsplit(".", 1)[0] + "-disamb.xml"
print "saving in :", new_f
open(new_f, 'w').write(codecs.encode(dom.toxml(), 'utf-8'))



# -*- coding: utf-8 -*-

import unittest
from wosedon.lesk_filters import No
import corpus2




def leniter(i):
  """ Lenght of any iterable """
  return sum(1 for e in i)


class TestLeskFilterNo(unittest.TestCase):

  def setUp(self):
    self.filter = No(corpus2.get_named_tagset('nkjp'), {})

  def test_it(self):
    self.assertNotIn('abcd', self.filter)
    self.assertNotIn('fgdesrg', self.filter)
    self.assertNotIn('ab24232cd', self.filter)
    self.assertNotIn('8ki67j', self.filter)
    self.assertNotIn('rtert', self.filter)
    self.assertNotIn('232 4 345', self.filter)






#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import sys
from collections import defaultdict

from wosedon.basegraph import BaseGraph
from mergerinterface import MergerInterface

class SynsetsMSRMerger(MergerInterface):
  """!
  Class for merging synsets graph and MSR graph.

  MSR graph was built based on the k-best file, which is described in MSRGraphBuilder class. 
  Each node represents one word, which consist of word lemma and part of speech. Whereas 
  each edge represents relation described by relatedness weight.

  The purpose of this class is to merge two graphs, namely synsets graph and 
  MSR graph. Nodes from MSR graph are conected to proper nodes from synsets graph, 
  which have them in their lexical units sets.
  """
  def __init__(self, resources, options, str_name = 'SynsetsMSRMerger'):
    """!
    @param resources - object of Resources class contain resources from config 
                       file
    @param options   - object of MergeOptions class contain merge options from 
                       config file
    @param str_name  - default value same as the class name "SynsetsMSRMerger"
    """
    super(SynsetsMSRMerger, self).__init__(resources, options, str_name)

  def merge(self, g1, g2):
    """!
    Merge two given graphs, namely synsets graph and MSR graph. The final 
    graph contain two types of nodes. Each synset node has an attribute named "synset", 
    to which is assigned synset object. Synset object consist of: synset_id and lu_set, 
    which contains lexical units objects. Whereas lexical units objects consists of: lu_id, 
    lemma, pos, variant and domain. Each MSR node has an attribute named "msr", to which is 
    assigned a word (word_lemma/POS). To the relation between synsets nodes is assigned 
    relation identifier from PLWN. Whereas to the relation between MSR nodes is assigned 
    weight, which describe a power of relatedness between linked words. New relations 
    are added between synsets nodes and MSR nodes. If word_lemma/POS of MSR node occur 
    in the object from lu_set of synset node then new relation is added between them.

    The directness of the graph and uniqueness of edges are inherited from synsets graph.

    Warning: If the final graph is a directed graph then two edges will be added between 
    a synset node and a MSR node. First edge will be add from synset node to MSR node 
    and second from MSR node to synset node.

    @param g1 - synsets graph
    @param g2 - MSR graph
    @return object of BaseGraph class
    """
    g = BaseGraph()
    g.init_graph(drctd = g1.is_directed())

    g.merge_graphs(g1, g2)

    lemma_on_vertex_dict = defaultdict(set)
    for node in g.all_nodes():
      if node.synset:
        lu_set = node.synset.lu_set
        for lu in lu_set:
          lemma_on_vertex_dict[lu.lemma + '/' + str(lu.pos)].add(node)

    for node in g.all_nodes():
      if node.msr:
        lemma_pos = node.msr
        if lemma_on_vertex_dict.has_key(lemma_pos):
          for synset_id_node in lemma_on_vertex_dict[lemma_pos]:
            g.add_edge(node, synset_id_node, [("rel", 'syn-msr')], simply = True)
            if g.is_directed():
              g.add_edge(synset_id_node, node, [("rel", 'syn-msr')], simply = True)
        else:
          logging.getLogger(__name__).warning("No synset in the graph contains lemma '%s'.", lemma_pos)

    return g



#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2015  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import operator
import rerankinterface

class NodeDegreeRanker(rerankinterface.RerankInterface):
  """Kazdy element rankingu (synset/wezel grafu) 
  wazony jest sopniem tego wezla."""

  def __init__(self, str_name = 'NodeDegreeRanker'):
    super(NodeDegreeRanker, self).__init__(str_name)

  def rerank(self, ranking, graph, options):
    new_rankg = []
    for (token, token_rank) in ranking:
      if not token_rank:
        new_rankg.append([token, token_rank])
        continue
      new_token_rank = []
      for (vertex, rank_val) in token_rank:
        deg = vertex.in_degree() + vertex.out_degree() if vertex else 1
        new_token_rank.append((vertex, rank_val * deg))
      new_rankg.append(
        (token, 
         sorted(
           new_token_rank, 
           key = operator.itemgetter(1), 
           reverse = True)))
    return new_rankg



# -*- encoding: utf-8 -*-
__author__ = 'nika'
import pickle

def count_ev(list_of_tuples):
    lll = []
    for ii in range(len(list_of_tuples[0])):
        lll.append(sum(map(lambda x: x[ii], list_of_tuples))/len(list_of_tuples))
    return lll

def count_ev_repr(list_of_tuples):
    lll = []
    for ii in range(len(list_of_tuples[0][0])):
        lll.append(sum(map(lambda x: x[0][ii], list_of_tuples))/len(list_of_tuples))
    for ii in range(len(list_of_tuples[0][1])):
        lll.append(sum(map(lambda x: x[1][ii], list_of_tuples))/len(list_of_tuples))
    return lll

def repr_stats(list_of_stats):
    all_sum = []
    for ii in range(len(list_of_stats[0])):
        all_sum.append(sum(map(lambda x:x[ii], list_of_stats)))
    result = [[1.0*all_sum[0]/all_sum[2], 1.0*all_sum[0]/all_sum[3], 1.0*all_sum[1]/all_sum[2], 1.0*all_sum[1]/all_sum[3], 1.0*all_sum[4]/all_sum[5]]]
    tmp = map(lambda x:[1.0*x[0]/x[2], 1.0*x[0]/x[3], 1.0*x[1]/x[2], 1.0*x[1]/x[3], 1.0*x[4]/x[5]], list_of_stats)
    all_sum = []
    for ii in range(len(tmp[0])):
        all_sum.append(sum(map(lambda x:x[ii], tmp))/len(tmp))
    result.append( all_sum )
    return result

class Tree(object):
    def __init__(self, path_):
        self.leaves = {} # słownik
        self.path = path_
        self.root = None
        self.nodes = {} # słownik id:node
        self.chosen_nodes = [] # tu tylko ids
        self.disamb_nodes = []
        self.chosen_dep_nodes = {} # tutaj (id, zależność)
        self.disamb_dep_nodes = {}
        self.mistakes = []
        self.nodes_prods = {}
        self.chosen_nodes_count = 0
        self.disamb_nodes_count = 0
        self.agreed_count = 0
        self.agreed_strict_count = 0
        self.dep_count = 0
        self.dep_agreed_count = 0
        self.missed_prods = []
        self.parents = {u'0':u'0'}
        self.parents_labels = {u'0':u'0'}
        self.heads = {}

    def hasRoot(self):
        if self.root:
            return True
        else:
            return False

    def addNode(self, node):
        self.nodes[node.getID()] = node
        self.nodes_prods[node.getID()] = []
        if not self.hasRoot():
            self.root = node

    def addChosenNode(self, node):
        self.addNode(node)
        self.chosen_nodes.append(node.getID())

    def addProd(self, node, prod):
        self.nodes_prods[node.getID()].append(prod)

    def addParent(self, node, children):
        node_id = node.getID()
        for child in children:
            if not child in self.parents:
                self.parents[child] = [node_id]
            else:
                self.parents[child].append(node_id)

    def addLeaf(self,id,info):
        self.leaves[id] = info
        
    def addHead(self, me, child):
        self.heads[me] = child
        
    
    def findHeads(self):
        all_leaves = False
        while not all_leaves:
            all_leaves = True
            for ii in self.heads:
                self.heads[ii] = self.heads[self.heads[ii]]
                if not self.nodes[self.heads[ii]].isTerminal():
                    all_leaves = False
        


    ################################ DEPENDENCY TREE MAKING

    def childIsChosen(self, child, tree):
        is_chosen = True
        for nid in child['productions']:
            if nid not in tree:
                is_chosen = False
        if child['centre'] and child['centre'] not in tree:
                is_chosen = False
        return is_chosen

    def get_centres(self, tree_from, chosen_tree_centres, deps, node_id):
        node = self.nodes[node_id]
        for child in node.getAllChildren():
            if self.childIsChosen(child, tree_from):
                #print node_id, child
                if child['centre']:
                    self.get_centres(tree_from, chosen_tree_centres, deps, child['centre'])
                    chosen_tree_centres[node_id] = chosen_tree_centres[child['centre']]
                for nid in child['productions']:
                    self.get_centres(tree_from, chosen_tree_centres, deps, nid)
                    deps.append((nid, node_id))
                if node.getTo() - node.getFrom() == 1:
                    chosen_tree_centres[node_id] = node.getTo()
        if node.isTerminal() and node.getTo() - node.getFrom() == 1:
            chosen_tree_centres[node_id] = node.getTo()

    def get_dep_tree(self, tree_from):
        chosen_tree_centres = {'sID':0}                    # tu będzie nid:najbl.centrum
        deps = [(self.root.getID(),'sID')]
        self.get_centres(tree_from, chosen_tree_centres, deps, self.root.getID())
        #print tree_from, chosen_tree_centres, deps
        return map(lambda x: (chosen_tree_centres[x[0]], chosen_tree_centres[x[1]]), deps)


    ################################ TREE DISAMBIGUATION TEMPLATE

    def disamb_tree_rec(self, select_best, node):
        if not node.isTerminal():
            pass
    def disamb_tree(self, select_best):
        pass

    ################################ TREE DISAMBIGUATION FUNCTIONS

    ##################### ACTUAL PCFG

    def getChildrenForActPcfg(self, prod_dict):
        ch = [(prod_dict['centre'], u'true')]
        ch += map(lambda x: (x,u'false'), prod_dict['productions'])
        return str(sorted(map(lambda x:(self.nodes[x[0]].getCategory(), x[1]), ch)))

    def act_pcfg_rec_count_probs(self, grammar, nodes_probs, node_id):
        #print "---------------------", nodes_probs
        if node_id in nodes_probs:
            return nodes_probs[node_id]
        else:
            node = self.nodes[node_id]
            poss_prods = node.getAllChildren()
            if not poss_prods:
                nodes_probs[node_id] = 1.0
                return nodes_probs[node_id]
            for prod_dict in poss_prods:
                prod_dict['prob'] = 1.0
                for ii in prod_dict['productions']:
                    prod_dict['prob'] *= self.act_pcfg_rec_count_probs(grammar, nodes_probs, ii)
                if 'centre' in prod_dict.keys():
                    prod_dict['prob'] *= self.act_pcfg_rec_count_probs(grammar, nodes_probs, prod_dict['centre'])
                try:
                    prod_dict['prob'] *= grammar[(node.getCategory(), str(map(lambda x:(self.nodes[x[0]].getCategory(), x[1]), prod_dict['whole_prod'])))]#self.getChildrenForActPcfg(prod_dict))]
                except KeyError:
                    prod_dict['prob'] *= min(map(lambda x: grammar[x], grammar.keys()))
                    self.missed_prods.append((node.getCategory(), str(map(lambda x:(self.nodes[x[0]].getCategory(), x[1]), prod_dict['whole_prod']))))
            nodes_probs[node_id] = sorted(poss_prods, key=lambda x: -x['prob'])[0]['prob']
            return nodes_probs[node_id]

    def act_pcfg_rec_get_tree(self, node_id):
        node = self.nodes[node_id]
        if node.isTerminal():
            self.disamb_nodes.append(node_id)
        else:
            poss_prods = node.getAllChildren()
            try:
                prod_dict = sorted(poss_prods, key=lambda x: -x['prob'])[0]
            except KeyError:
                prod_dict = poss_prods[0]
            self.disamb_nodes.append( node_id )
            if 'centre' in prod_dict.keys():
                self.act_pcfg_rec_get_tree(prod_dict['centre'])
            for prod in prod_dict['productions']:
                self.act_pcfg_rec_get_tree(prod)

    def act_pcfg(self, grammar):
        nodes_probs = {}
        self.disamb_nodes = []
        self.disamb_dep_nodes = {}
        self.act_pcfg_rec_count_probs(grammar, nodes_probs, self.root.getID())
        self.act_pcfg_rec_get_tree(self.root.getID())
        #self.chosen_dep_nodes = self.get_dep_tree(self.chosen_nodes)
        #self.disamb_dep_nodes = self.get_dep_tree(self.disamb_nodes)
        return self.getStats()#self.precision(), self.recall(), self.precisionStrict(), self.recallStrict(), self.precisionDep()

    def getMissedProds(self):
        res = {}
        for ii in set(self.missed_prods):
            res[ii] = self.missed_prods.count(ii)
        return res


    ################################ EVALUATION
    def no_of_nonterminals(self, tree):
        count = 0
        for ii in tree:
            if not self.nodes[ii].isTerminal():
                count += 1
        return count

    def getStats(self):
        agreed = 0
        for node1 in self.chosen_nodes:
            for node2 in self.disamb_nodes:
                if self.nodes[node1].equals_from_to_cat(self.nodes[node2]) and not self.nodes[node1].isTerminal():
                    agreed += 1
        self.agreed_count = agreed
        agreed = 0
        for node1 in self.chosen_nodes:
            for node2 in self.disamb_nodes:
                if self.nodes[node1].equals(self.nodes[node2]) and not self.nodes[node1].isTerminal():
                    agreed += 1
        self.agreed_strict_count = agreed
        self.chosen_nodes_count = self.no_of_nonterminals(self.chosen_nodes)
        self.disamb_nodes_count = self.no_of_nonterminals(self.disamb_nodes)
        agreed = 0
        for node in self.chosen_dep_nodes:
            if node in self.disamb_dep_nodes:
                agreed += 1
        self.dep_agreed_count = agreed
        self.dep_count = len(self.chosen_dep_nodes)
        return self.agreed_count, self.agreed_strict_count, self.chosen_nodes_count, self.disamb_nodes_count, self.dep_agreed_count, self.dep_count

    def precision(self):
        agreed = 0
        #print sorted(self.chosen_nodes)
        #print sorted(self.disamb_nodes)
        for node1 in self.chosen_nodes:
            #print "*", node1
            for node2 in self.disamb_nodes:
                if self.nodes[node1].equals_from_to_cat(self.nodes[node2]) and not self.nodes[node1].isTerminal():
                    agreed += 1
                    #break
                    #print "#", node2
        self.agreed_count = agreed
        self.chosen_nodes_count = self.no_of_nonterminals(self.chosen_nodes)
        return 1.0 * agreed / self.no_of_nonterminals(self.chosen_nodes)

    def recall(self):
        agreed = 0
        for node1 in self.chosen_nodes:
            for node2 in self.disamb_nodes:
                if self.nodes[node1].equals_from_to_cat(self.nodes[node2]) and not self.nodes[node1].isTerminal():
                    agreed += 1
                    #break
        return 1.0 * agreed / self.no_of_nonterminals(self.disamb_nodes)

    def precisionStrict(self):
        agreed = 0
        for node1 in self.chosen_nodes:
            for node2 in self.disamb_nodes:
                if self.nodes[node1].equals(self.nodes[node2]) and not self.nodes[node1].isTerminal():
                    agreed += 1
                    break
        return 1.0 * agreed / self.no_of_nonterminals(self.chosen_nodes)

    def recallStrict(self):
        agreed = 0
        for node1 in self.chosen_nodes:
            for node2 in self.disamb_nodes:
                if self.nodes[node1].equals(self.nodes[node2]) and not self.nodes[node1].isTerminal():
                    agreed += 1
                    break
        return 1.0 * agreed / self.no_of_nonterminals(self.disamb_nodes)

    def precisionDep(self):
        agreed = 0
        for node in self.chosen_dep_nodes:
            if node in self.disamb_dep_nodes:
                agreed += 1
        return 1.0 * agreed / len(self.chosen_dep_nodes)

    def getProperChildren(self, node_id, tree):
        node = self.nodes[node_id]
        for child in node.getAllChildren():
            if self.childIsChosen(child, tree):
                return child
        return {}

    def equals_from_to_cat(self, node1, node2):
        return self.nodes[node1].equals_from_to_cat(self.nodes[node2])

    def findMistakes_rec(self, node_id):
        for ii in self.disamb_nodes:
            if self.equals_from_to_cat(node_id, ii):
                child1 = self.getProperChildren(node_id, self.chosen_nodes)
                child2 = self.getProperChildren(ii, self.disamb_nodes)
                if self.equals_from_to_cat(child1['centre'], child2['centre']):
                    if not self.nodes[child1['centre']].isTerminal():
                        self.findMistakes_rec(child1['centre'])
                else:
                    self.mistakes.append(('centre', self.nodes[child1['centre']].getCategory(), self.nodes[child2['centre']].getExtCategory()))
                for jj in child1['productions']:
                    for kk in child2['productions']:
                        if self.equals_from_to_cat(jj,kk):
                            if not self.nodes[jj].isTerminal():
                                self.findMistakes_rec(jj)
                        else:
                            if self.nodes[jj].overlaps(self.nodes[kk]):
                                self.mistakes.append(('noncentre', self.nodes[jj].getCategory(), self.nodes[kk].getExtCategory()))

    def findMistakes(self):
        self.mistakes = []
        self.findMistakes_rec(self.root.getID())
        return self.mistakes

    def findMistakesSimpleRec(self, node_id):
        children = self.getProperChildren(node_id, self.disamb_nodes)
        for ii in self.disamb_nodes:
            if self.equals_from_to_cat(node_id, ii):
                child1 = self.getProperChildren(node_id, self.chosen_nodes)
                child2 = self.getProperChildren(ii, self.disamb_nodes)
                if not self.equals_from_to_cat(child1['centre'], child2['centre']):
                    self.mistakes.append(('centre', self.nodes[child1['centre']].getCategory(), self.nodes[child2['centre']].getExtCategory()))
                if not self.nodes[child1['centre']].isTerminal():
                    self.findMistakes_rec(child1['centre'])
                for jj in child1['productions']:
                    for kk in child2['productions']:
                        if self.equals_from_to_cat(jj,kk):
                            if not self.nodes[jj].isTerminal():
                                self.findMistakes_rec(jj)
                        else:
                            if self.nodes[jj].overlaps(self.nodes[kk]):
                                self.mistakes.append(('noncentre', self.nodes[jj].getCategory(), self.nodes[kk].getExtCategory()))


    def findMistakesSimple(self):
        self.mistakes = []
        #self.findMistakesSimple_rec(self.root.getID())
        for node_id in self.disamb_nodes:
            node = self.nodes[node_id]
            children = self.getProperChildren(node_id, self.disamb_nodes)
            #print children
            if children:
                for child in children['productions']:
                    has = False
                    for ii in self.chosen_nodes:
                        if self.equals_from_to_cat(child, ii):
                            has = True
                    if not has:
                        self.mistakes.append(('noncentre', node.getCategory(), self.nodes[child].getCategory()))
                has = False
                for ii in self.chosen_nodes:
                    if self.equals_from_to_cat(children['centre'], ii):
                        has = True
                if not has:
                    self.mistakes.append(('centre', node.getCategory(), self.nodes[children['centre']].getCategory()))
        #print self.mistakes
        return self.mistakes

    def countChildrenZadnie(self):
        res = []
        for node_id in self.chosen_nodes:
            for ii in self.disamb_nodes:
                if self.nodes[node_id].getCategory() == u"zdanie" and self.equals_from_to_cat(node_id, ii):
                    child1 = len(self.getProperChildren(node_id, self.chosen_nodes)['productions'])+int('centre' in self.getProperChildren(node_id, self.chosen_nodes))
                    child2 = len(self.getProperChildren(ii, self.disamb_nodes)['productions'])+int('centre' in self.getProperChildren(ii, self.disamb_nodes))
                    #print self.getProperChildren(node_id, self.chosen_nodes)['whole_prod'], self.getProperChildren(node_id, self.disamb_nodes)['whole_prod']
                    res.append((child1, child2))
        return res

    def getDisambNodes(self):
        return self.disamb_nodes



# -*- encoding: utf-8 -*-
__author__ = 'nika'

from xml.sax import handler, make_parser
from multiprocessing import Pool

class ProductionParserActPCFG(handler.ContentHandler):

    def __init__(self):
        self.current_tag = ""
        self.nodes = {}
        self.productions = []
        self.productions_dict = {}
        self.in_chosen_node = False
        self.nid = None
        self.in_chosen_children = False
        self.current_prod = []
        self.full_tree = False
        self.prod_with_centre = []
        self.is_in_fw = False
        self.is_in_l = False
        self.is_in_o = False
        self.is_in_r = False
        self.is_in_p = False
        self.is_in_tfw = False

    def getProductions(self):
        for prod in self.productions_dict:
            pass#print prod, self.productions_dict[prod]
        return self.productions_dict

    def startElement(self, name, attrs):
        self.current_tag = name
        if name == "node" and attrs.getValue("chosen") == "true":
            self.in_chosen_node = True
            self.nid = attrs.getValue("nid")
        if name == "children" and "chosen" in attrs.getNames() and attrs.getValue("chosen") == "true":
            self.in_chosen_children = True
        if name == "child" and self.in_chosen_children:
            self.current_prod.append(( attrs.getValue("nid"), attrs.getValue("head") ))
        if name == "terminal" and self.in_chosen_node:
            self.nodes[self.nid] = u'terminal'
        if name == "f" and self.in_chosen_node:
            if attrs.getValue("type") == "tfw" and self.is_in_fw:
                self.is_in_tfw = True
            elif attrs.getValue("type") == "liczba":
                self.is_in_l = True
            elif attrs.getValue("type") == "osoba":
                self.is_in_o = True
            elif attrs.getValue("type") == "rodzaj":
                self.is_in_r = True
            elif attrs.getValue("type") == "przypadek":
                self.is_in_p = True


    def endElement(self, name):                         # zlicza w zależności od węzła
        if name == "node":
            self.in_chosen_node = False
        if name == "children" and self.in_chosen_children:
            self.in_chosen_children = False
            self.productions.append(( self.nid, self.current_prod ))
            self.current_prod = []

    def characters(self, content):
        if self.current_tag == "category" and self.in_chosen_node and content.strip():
            self.nodes[self.nid] = content.strip()
            if content.strip() == "fw":
                self.is_in_fw = True
        if self.current_tag == "f" and content.strip() and self.in_chosen_node:
            if self.is_in_tfw:
                self.nodes[self.nid] += '@' + content.strip()
                self.is_in_fw = False
                self.is_in_tfw = False
            elif self.is_in_l:
                #self.nodes[self.nid] += '@' + content.strip()
                self.is_in_l = False
            elif self.is_in_o:
                #self.nodes[self.nid] += '@' + content.strip()
                self.is_in_o = False
            elif self.is_in_r:
                #self.nodes[self.nid] += '@' + content.strip()
                self.is_in_r = False
            elif self.is_in_p:
                #self.nodes[self.nid] += '@' + content.strip()
                self.is_in_p = False


    def endDocument(self):
        tmp_prod = map(lambda x: (self.nodes[x[0]], map(lambda y: (self.nodes[y[0]], y[1]), x[1] )), self.productions)
        for ii in list(set(map(lambda x: (x[0],str(x[1])),tmp_prod))):
            self.productions_dict[ii] = map(lambda x: (x[0],str(x[1])),tmp_prod).count(ii)

def runParse(fname):
    parser = make_parser()
    handler = ProductionParserActPCFG()
    parser.setContentHandler(handler)
    parser.parse(fname)
    return handler.getProductions()

def makeActPcfg(flist, tasks=4):
    """

    """
    pool = Pool(processes=tasks)
    all_prods = {}
    for productions in pool.map(runParse, flist):
        for production in productions:
            if production in all_prods:
                all_prods[production] += productions[production]
            else:
                all_prods[production] = productions[production]

    non_term_count = {}
    prods_prob = {}
    #print all_prods
    non_term = map(lambda x: x[0], all_prods)
    for ii in set(non_term):
        non_term_count[ii] = 0
        for jj in all_prods:
            if jj[0] == ii:
                non_term_count[ii] += all_prods[jj]
    for ii in all_prods:
        prods_prob[ii] = all_prods[ii] * 1.0 / non_term_count[ii[0]]

    return prods_prob


#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import os, sys, logging
import corpus2
from collections import defaultdict

class Resources:
  def __init__(self):
    # corpus2 tagset object
    self._tagset = None

    self._msr_file = None
    
    self._plwn_graph_file = None

    self._wnd_graph_file = None
    self._sumo_graph_file = None
    self._ccl_graph_file = None

    self._mapping_sumo_file = None
    self._mapping_wnd_file = None

    self._gloss_file = None
    self._gloss_rel_file = None
    
    self._impedance_table = None

  def tagset(self):
    if not isinstance(self._tagset, corpus2.Tagset):
      self._tagset = corpus2.get_named_tagset(self._tagset)
    return self._tagset

  def msr_file(self):
    return self._msr_file
  
  def plwn_graph_file(self):
    return self._plwn_graph_file

  def wnd_graph_file(self):
    return self._wnd_graph_file

  def sumo_graph_file(self):
    return self._sumo_graph_file

  def ccl_graph_file(self):
    return self._ccl_graph_file

  def mapping_wnd_file(self):
    return self._mapping_wnd_file

  def mapping_sumo_file(self):
    return self._mapping_sumo_file

  def gloss_file(self):
    if isinstance(self._gloss_file, str):
      self._prepare_gloss_dict()
    return self._gloss_file

  def gloss_rel_file(self):
    return self._gloss_rel_file
  
  def impedance_table(self):
    if not isinstance(self._impedance_table, defaultdict):
      it = defaultdict(lambda: defaultdict(float))
      with open(self._impedance_table) as f:
        headers = f.readline().strip().split(',')
        headers = [h.strip() for h in headers]
        
        for l in f:
          l = l.strip().split(',')
          row = l[0].strip()
          it[row] = defaultdict(float)
          for i in xrange(1, len(headers)):
            it[row][headers[i]] = float(l[i])
      
      self._impedance_table = it
    return self._impedance_table

  def set_tagset(self, tagset):
    self._tagset = tagset

  def set_msr_file(self, msr_file):
    self._msr_file = msr_file

  def set_plwn_graph_file(self, pgf):
    self._plwn_graph_file = pgf
  
  def set_wnd_graph_file(self, wgf):
    self._wnd_graph_file = wgf
  
  def set_sumo_graph_file(self, sgf):
    self._sumo_graph_file = sgf
  
  def set_ccl_graph_file(self, cgf):
    self._ccl_graph_file = cgf

  def set_mapping_sumo_file(self, msf):
    self._mapping_sumo_file = msf

  def set_mapping_wnd_file(self, mwf):
    self._mapping_wnd_file = mwf

  def set_gloss_file(self, gf):
    self._gloss_file = gf

  def set_gloss_rel_file(self, grf):
    self._gloss_rel_file = grf
  
  def set_impedance_table(self, path):
    self._impedance_table = path

  def _prepare_gloss_dict(self):
    if not os.path.exists(self._gloss_file):
      raise IOError(
        "%s file not found!" % \
          self._gloss_file)

    try:
      cclreader = corpus2.CclRelReader(
                      self.tagset(), 
                      self._gloss_file, 
                      self._gloss_file
                  )
      document = cclreader.read()
      self._gloss_file = {}
      for paragraph in document.paragraphs():
        synset_id = int(paragraph.get_attribute('id'))
        self._gloss_file[synset_id] = paragraph
    except Exception, e:
      logging.getLogger(__name__).exception('Cannot load gloss file.')
      raise



from wosedon.lesk_functions.leskfunctioninterface import LeskFunctionInterface
from collections import deque

  
def _mult_and_add_to(dict1, dict2, c):
  for k in dict2:
    if k in dict1:
      dict1[k] += dict2[k] * c
    else:
      dict1[k] = dict2[k] * c



class DictionaryBased(LeskFunctionInterface):
  """!
  Prepares dictionaries {(lemma, pos) : importance} both for context
  and definition.
  """
  
  def __init__(self, resources, filter_obj, graph, options, str_name):
    super(DictionaryBased, self).__init__(resources, filter_obj, graph, options, str_name)
    
    self.definition_cache = {}
    self.damping = self.options.damping_factor()
    self.iter = self.options.max_iter()
    
  
  
  def _get_index(self, token):
    """!
    Convert token into pair (lemma, part of speech), which can be used as index
    in being prepared dictionaries.
    """
    lexeme = token.get_preferred_lexeme(self.tagset)
    if lexeme in self.filter_obj:
      lemma = str(lexeme.lemma())
      pos_str = self.tagset.get_pos_name(lexeme.tag().get_pos_index())
      return lemma, pos_str
    return None


  def _prepare_definition(self, synset_id):
    if not synset_id in self.definition_cache:
      if synset_id in self.gloss_dict:
        definition = self.gloss_dict[synset_id]
        res = {}
        for sentence in definition.sentences():
          for token in sentence.tokens():
            ind = self._get_index(token)
            res[ind] = 1.0
              
        if None in res:
          del res[None]
        self.definition_cache[synset_id] = res
      else:
        self.definition_cache[synset_id] = {}
    return self.definition_cache[synset_id]
  
  
  def prepare_node(self, node):
    res = {}
    
    q = deque()
    visited = set()
    
    q.append((node, 0, 1.0))
    visited.add(node)
    
    while q:
      v, d, w = q.popleft()
      
      _mult_and_add_to(res,  self._prepare_definition(int(v.synset.synset_id)), w)
      
      if d < self.iter:
        for e in v.all_edges():
          if e.target() not in visited:
            q.append((e.target(), d+1, w * e.weight * self.damping))
            visited.add(e.target())
    
    return res
  
    
  def prepare_context(self, context):
    res = {}
    
    for token in context.tokens():
      ind = self._get_index(token)
      if ind in res:
          res[ind] += 1.0
      else:
          res[ind] = 1.0
    
    if None in res:
      del res[None]
    return res















from wosedon.lesk_functions.dictionarybased import DictionaryBased



class ExampleFunction(DictionaryBased):
  
  def __init__(self, resources, filter_obj, graph, options, str_name='ExampleFunction'):
    super(ExampleFunction, self).__init__(resources, filter_obj, graph, options, str_name)
    
  
  def compare(self, definition_data, context_data):
    coverage = 0.0
    for (lemma, pos_str) in definition_data:
      if (lemma, pos_str) in context_data:
        coverage -= abs(definition_data[(lemma, pos_str)] - context_data[(lemma, pos_str)])**1.2
    return coverage
    



import sys
from wosedon.reranks.rerankinterface import RerankInterface

class LemmaRankingFirstSelecter(RerankInterface):
  """Dla lematow sprawdzana jest roznica pomiedzy wartoscia 
  rankingu pierwszego i drugiego synsetu. Jezeli
  roznica jest mniejsza niz zadana w pliku konfiguracyjnym
  wartosc, to do lematu przypisywany jest synset, ktory ma 
  najmiejszy wariant."""

  def __init__(self, str_name = 'LemmaRankingFirstSelecter'):
    super(LemmaRankingFirstSelecter, self).__init__(str_name)

  def rerank(self, ranking, graph, options):
    tagset = options.tagset()

    new_ranking = []
    for (token, token_rank) in ranking:
      if not token_rank:
        new_ranking.append([token, token_rank])
        continue

      if len(token_rank) == 1:
        new_ranking.append([token, token_rank])
        continue

      lemma = str(token.get_preferred_lexeme(tagset).lemma())

      rank_sum = sum(rank for (node, rank) in token_rank)
      percentage_diff_value = (float(options.percentage_diff()) * float(rank_sum)) / 100.0

      first_synset_rank = token_rank[0][1]
      second_synset_rank = token_rank[1][1]
      diff = abs(first_synset_rank - second_synset_rank)

      if diff <= percentage_diff_value:
        selected_element = self.get_element_of_the_highest_variant(token_rank, lemma, graph)
        token_rank.remove(selected_element)
        token_rank.insert(0, selected_element)

      new_ranking.append((token, token_rank))

    return new_ranking

  def get_element_of_the_highest_variant(self, token_rank, lemma, graph):
    element = None
    variant = sys.maxint
    for (node, rank) in token_rank:
      for lu in node.synset.lu_set:
        if lu.lemma == lemma:
          temp_variant = lu.variant
          break
      if temp_variant < variant:
        element = (node, rank)
        variant = temp_variant
    return element



#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2015  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import operator
from wosedon.reranks.rerankinterface import RerankInterface

class LemmaRankingNormalizer(RerankInterface):
  """Normalizuje ranking dla lematu do przedzialu <0,1>.
  Ranking kazdego lematu (de facto tokenu) normalizoweny jest
  do przedzialu <0,1>"""
  
  def __init__(self, str_name = 'LemmaRankingNormalizer'):
    super(LemmaRankingNormalizer, self).__init__(str_name)

  def rerank(self, ranking, graph, options):
    new_rankg = []
    for (token, token_rank) in ranking:
      if not token_rank:
        new_rankg.append([token, token_rank])
        continue
      new_token_rank = []
      node_rank_sum = sum(float(n[1]) for n in token_rank)
      for (vertex, rank_val) in token_rank:
        new_token_rank.append((
          vertex, 
          float(rank_val) / node_rank_sum if node_rank_sum else 1.0))
      new_rankg.append((token, new_token_rank))
    return new_rankg



from wosedon.algorithms.wsdalgorithminterface import WSDAlgorithmInterface
from wosedon.ranking.wsd_ranking import WSDRanking
from collections import deque, defaultdict


class PaintballWSD(WSDAlgorithmInterface):

  def __init__(self, str_name = 'PaintballWSD'):
    """!
    @param str_name - default value same as the class name "PaintballWSD"
    """
    super(PaintballWSD, self).__init__(str_name)


  def prepare_v(self, wsd_context, graph):
    """!
    Required by WSDAlgorithmInterface.
    """
    pass
  
  
  def make_original_values(self, lemma_on_node_dict):
    vals = {}
    for nodes in lemma_on_node_dict.itervalues():
      for node in nodes:
        if node in vals:
          vals[node] += self._multiply_factor / float(len(nodes))
        else:
          vals[node] = self._multiply_factor / float(len(nodes))
    return vals
  
  
  def BFS(self, start, original_value, values, impedance_table):
    Q = deque()
    values[start] += original_value
    visited = set()
    visited.add(start)
    
    for edge in start.all_edges():
      if edge.target() != start:
        Q.append((edge, original_value * edge.weight * self.damping, 1))
        visited.add(edge.target())
    
    while Q:
      e, v, d = Q.popleft()
      values[e.target()] += v
      if d < self.iter:
        for e2 in e.target().all_edges():
          if e2.target() not in visited:
            Q.append((e2, v * self.damping * e2.weight * self.impedance_table[e.rel][e2.rel], d + 1))
            visited.add(e2.target())
      
  
  
  def run(self, wsd_context, graph, options, resources):
    """!
    Disambiguate analyzed context.

    @param wsd_context - object of WSDContext class
    @param graph - object of BaseGraph class
    @param options - object of AlgorithmOptions class
    @param resources - object of Resources class
    @return tuple of object of WSDRanking class and number of algorithm iterations
    """
    self.impedance_table = resources.impedance_table()
    self.iter = options.max_iter()
    self.damping = options.damping_factor()
    
    
    wsd_rank = WSDRanking()
    (lemma_on_node_dict, lemma_on_only_synset_node_dict) = \
      wsd_rank.get_lemma_on_node_dict(wsd_context, graph, options.ini_nodes())
    
    original_values = self.make_original_values(lemma_on_node_dict)
    values = defaultdict(float)
    
    for node, original_value in original_values.iteritems():
      self.BFS(node, original_value, values, resources.impedance_table)

    for (lemma, pos_str) in lemma_on_only_synset_node_dict.iterkeys():
      wsd_rank.set_ranking_for_lemma(lemma, pos_str, values)

    return (wsd_rank, self.iter)
    



import sys

from vertices.lexical_unit import LexicalUnit
from vertices.synset import Synset

from graph_tool import Graph, load_graph


class PLWNGraph:
  def __init__(self):
    self.dbconnection = None

    self.syn_G = Graph()
    self.lu_G = Graph()

    self._lexicon_dict = None
    self._lu_dict = None
    self._syn_dict = None
    
    self._syn_on_vertex_dict = None
    self._lu_on_vertex_dict = None

  def _get_lexicon_dict(self):
    lexicon_dict = {}

    sql_query = 'SELECT DISTINCT LEX.id, T.text ' \
                'FROM lexicon LEX JOIN text T on (LEX.name = T.id);'

    cursor = self.dbconnection.cursor()
    cursor.execute(sql_query)

    for row in cursor.fetchall():
      lexicon_id = int(row[0])
      lexicon_name = str(row[1]).replace(' ', '_')
      lexicon_dict[lexicon_id] = lexicon_name

    return lexicon_dict

  def _get_lu_dict(self):
    '''
    {lu_id_1: lu_object_1, lu_id_2: lu_object_2, ..., lu_id_N: lu_object_N}
    {..., 46469: <PLWNGraphBuilder.lexical_unit.LexicalUnit instance at 0x47a4878>, ...}
    '''
    lu_dict = {}

    sql_query = 'SELECT DISTINCT SEN.id, W.word, SEN.domain, ' \
                                'SEN.part_of_speech, SEN.sense_number, ' \
                                'SEN.id_lexicon ' \
                'FROM sense SEN JOIN word W ON (SEN.lemma = W.id);'

    cursor = self.dbconnection.cursor()
    cursor.execute(sql_query)

    for row in cursor.fetchall():
      lu_id = int(row[0])
      lu_lemma = str(row[1])
      lu_domain = int(row[2])
      lu_pos = int(row[3])
      lu_variant = int(row[4])
      lu_lexicon = self._lexicon_dict[int(row[5])]

      lu = LexicalUnit(lu_id, lu_lemma, lu_pos, lu_domain, lu_variant, lu_lexicon)

      if not lu_dict.has_key(lu_id):
        lu_dict[lu_id] = lu
      else:
        print >> sys.stderr, 'W bazie danych Slowosieci powtorzyl sie', \
                             'identyfikator dla roznych jednostek leksykalnych!', \
                             '(' + lu_id + ')'

    return lu_dict

  def _get_syn_dict(self):
    '''
    {syn_id_1: syn_object_1, syn_id_2: syn_object_2, ..., syn_id_N: syn_object_N}
    {..., 30421: <PLWNGraphBuilder.synset.Synset instance at 0x82b1170>, ...}
    '''
    syn_dict = {}
    
    sql_query = 'SELECT DISTINCT id ' \
                'FROM synset;'

    cursor = self.dbconnection.cursor()
    cursor.execute(sql_query)

    for row in cursor.fetchall():
      synset_id = int(row[0])

      lu_set = set()
      lu_set = self._get_lu_set(synset_id)

      syn = Synset(synset_id, lu_set)
      syn_dict[synset_id] = syn

    return syn_dict

  def _get_lu_set(self, synset_id):
    '''
    Dla podanego na wejsciu identyfikatora synsetu
    zwraca zbior obiektow jednostek leksykalnych,
    ktore sie w nim zawieraja.
    '''
    lu_set = set()
    
    sql_query = 'SELECT DISTINCT id_sense ' \
                'FROM sense_to_synset ' \
                'WHERE id_synset=' + str(synset_id) + ';'

    cursor = self.dbconnection.cursor()
    cursor.execute(sql_query)

    for row in cursor.fetchall():
      lu_id = int(row[0])

      if self._lu_dict.has_key(lu_id):
        lu_set.add(self._lu_dict[lu_id])
      else:
        print >> sys.stderr, 'W tabeli lexicalunit brakuje jednostki leksykalnej', lu_id, \
                             ', ktora pojawila sie w tabeli unitandsynset!'
    
    return lu_set

  # Dodawanie wierzcholkow do grafow.
  def _add_syn_vertices(self):
    '''
    {synset_id_1: vertex_id_1, synset_id_2: vertex_id_2, ..., synset_id_N: vertex_id_N}
    {..., 10: 0, 11: 1, ...}
    '''
    self._syn_on_vertex_dict = {}

    v_synset = self.syn_G.new_vertex_property("python::object")
    for synset_id, syn in self._syn_dict.iteritems():
      v = self.syn_G.add_vertex()
      v_synset[v] = syn
      self._syn_on_vertex_dict[synset_id] = v

    self.syn_G.vertex_properties["synset"] = v_synset

  def _add_lu_vertices(self):
    '''
    {lu_id_1: vertex_id_1,lu_id_2: vertex_id_2, ..., lu_id_N: vertex_id_N}
    {..., 11: 0, 12: 1, ...}
    '''
    self._lu_on_vertex_dict = {}

    v_lu = self.lu_G.new_vertex_property("python::object")
    for lu_id, lu in self._lu_dict.iteritems():
      v = self.lu_G.add_vertex()
      v_lu[v] = lu
      self._lu_on_vertex_dict[lu_id] = v

    self.lu_G.vertex_properties["lu"] = v_lu

  # Dodawanie krawedzi do grafow.
  def _add_syn_edges(self):
    e_rel_id = self.syn_G.new_edge_property("int")

    sql_query = 'SELECT DISTINCT synset_from, synset_to, relation ' \
                'FROM synset_relation;'

    cursor = self.dbconnection.cursor()
    cursor.execute(sql_query)

    for row in cursor.fetchall():
      parent_id = int(row[0])
      child_id = int(row[1])
      rel_id = int(row[2])
      
      v_parent_id = None
      if self._syn_on_vertex_dict.has_key(parent_id):
        v_parent_id = self._syn_on_vertex_dict[parent_id]
      else:
        print >> sys.stderr, 'Synset rodzic', parent_id, 'pojawil sie w tabeli ' \
                             'synsetrelation, a brakuje go w tabeli synset!'
        continue
      
      v_child_id = None
      if self._syn_on_vertex_dict.has_key(child_id):
        v_child_id = self._syn_on_vertex_dict[child_id]
      else:
        print >> sys.stderr, 'Synset dziecko', child_id, 'pojawil sie w tabeli ' \
                             'synsetrelation, a brakuje go w tabeli synset!'
        continue
 
      e = self.syn_G.add_edge(v_parent_id, v_child_id)
      e_rel_id[e] = rel_id

    self.syn_G.edge_properties["rel_id"] = e_rel_id

  def _add_lu_edges(self):
    e_rel_id = self.lu_G.new_edge_property("int")

    sql_query = 'SELECT DISTINCT sense_from, sense_to, relation ' \
                'FROM sense_relation;'

    cursor = self.dbconnection.cursor()
    cursor.execute(sql_query)

    for row in cursor.fetchall():
      parent_id = int(row[0])
      child_id = int(row[1])
      rel_id = int(row[2])
      
      v_parent_id = None
      if self._lu_on_vertex_dict.has_key(parent_id):
        v_parent_id = self._lu_on_vertex_dict[parent_id]
      else:
        print >> sys.stderr, 'Synset rodzic', parent_id, 'pojawil sie w tabeli ' \
                             'lexicalrelation, a brakuje go w tabeli lexicalunit!'
        continue
      
      v_child_id = None
      if self._lu_on_vertex_dict.has_key(child_id):
        v_child_id = self._lu_on_vertex_dict[child_id]
      else:
        print >> sys.stderr, 'Synset dziecko', child_id, 'pojawil sie w tabeli ' \
                             'lexicalrelation, a brakuje go w tabeli lexicalunit!'
        continue
      
      e = self.lu_G.add_edge(v_parent_id, v_child_id)
      e_rel_id[e] = rel_id

    self.lu_G.edge_properties["rel_id"] = e_rel_id


  def build_graphs(self, dbconnection):
    self.dbconnection = dbconnection

    self._lexicon_dict = self._get_lexicon_dict()
    self._lu_dict = self._get_lu_dict()
    self._syn_dict = self._get_syn_dict()

    print >> sys.stderr, 'Add syn vertices...',
    self._add_syn_vertices()
    print >> sys.stderr, 'Done!'
    print >> sys.stderr, 'Add lu vertices...',
    self._add_lu_vertices()
    print >> sys.stderr, 'Done!'
    
    print >> sys.stderr, 'Add syn edges...',
    self._add_syn_edges()
    print >> sys.stderr, 'Done!'
    print >> sys.stderr, 'Add lu edges...',
    self._add_lu_edges()
    print >> sys.stderr, 'Done!'
  
  def save_graphs(self, out_graphs_file):
    print >> sys.stderr, 'Save syn graph to file...',
    path_to_syn_graph = out_graphs_file + '_syn.xml.gz'
    self.syn_G.save(path_to_syn_graph)
    print >> sys.stderr, 'Done!'

    print >> sys.stderr, 'Save lu graph to file...',
    path_to_lu_graph = out_graphs_file + '_lu.xml.gz'
    self.lu_G.save(path_to_lu_graph)
    print >> sys.stderr, 'Done!'
  
  def load_syn_graph(self, in_syn_graph_file):
    print >> sys.stderr, 'Load syn graph from file...',
    self.syn_G = load_graph(in_syn_graph_file)
    print >> sys.stderr, 'Done!'

  def load_lu_graph(self, in_lu_graph_file):
    print >> sys.stderr, 'Load lu graph from file...',
    self.lu_G = load_graph(in_lu_graph_file)
    print >> sys.stderr, 'Done!'


#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

from graph_tool.centrality import pagerank
from wosedon.algorithms.wsdalgorithminterface import WSDAlgorithmInterface
from wosedon.ranking.wsd_ranking import WSDRanking

class GTPersonalizedW2WPR(WSDAlgorithmInterface):
  """!
  Personalized Word-to-Word Page Rank algorithm based on the set of synsets 
  which correspond to lemmas from the analyzed context. Each word (lemma) from
  analyzed context is separately disambiguated. All synsets which belong to 
  lemmas from the analyzed context, excluding synsets which belong to 
  disambiguated word (lemma), have the same value in personalized vector "v", 
  which is equal to 1/N, where N is the number of synsets which do not 
  correspond to disambiguated word (lemma). Remaining synsets which do not
  belong to lemmas from the analyzed context and synsets which belong to 
  disambiguated word (lemma) have value equal to 0 in personalized vector "v".
  """

  def __init__(self, str_name = 'GTPersonalizedW2WPR'):
    """!
    @param str_name - default value same as the class name "GTPersonalizedW2WPR"
    """
    super(GTPersonalizedW2WPR, self).__init__(str_name)
    self._context_node_set = None

  def prepare_v(self, wsd_context, graph):
    """!
    Create personalized vector "v".

    All synsets which belong to lemmas from the analyzed context, excluding 
    synsets which belong to disambiguated word (lemma), have the same value 
    in personalized vector "v", which is equal to 1/N, where N is the number 
    of synsets which do not correspond to disambiguated word (lemma).

    Remaining synsets which do not belong to lemmas from the analyzed context
    and synsets which belong to disambiguated word (lemma) have value equal to
    0 in personalized vector "v".

    @param wsd_context - object of WSDContext class
    @param graph - object of BaseGraph class
    @return PropertyMap (see: http://graph-tool.skewed.de/static/doc/graph_tool.html?highlight=new_vertex_property#graph_tool.Graph.new_vertex_property) of personalized vector "v"
    """
    u = graph.use_graph_tool().new_vertex_property("double")
    for node in self._context_node_set:  # all others have 0.0 by default
      u[node.use_graph_tool()] = self._multiply_factor * (1.0 / float(len(self._context_node_set)))
      
    return u

  def run(self, wsd_context, graph, options, resources):
    """!
    Disambiguate analyzed context.

    @param wsd_context - object of WSDContext class
    @param graph - object of BaseGraph class
    @param options - object of AlgorithmOptions class
    @param resources - object of Resources class
    @return tuple of object of WSDRanking class and number of algorithm iterations
    """
    ret_iter_list = []

    wsd_rank = WSDRanking()
    (lemma_on_node_dict, lemma_on_only_synset_node_dict) = \
      wsd_rank.get_lemma_on_node_dict(wsd_context, graph, options.ini_nodes())

    for analyzed_lemma, nodes in lemma_on_node_dict.iteritems():
      self._context_node_set = set()
      self._set_context_node_set(analyzed_lemma, lemma_on_node_dict)

      pers_v = self.prepare_v(wsd_context, graph)
      (ranking, ret_iter) = pagerank(graph.use_graph_tool(), 
                                     pers = pers_v, 
                                     max_iter = 2 * options.max_iter(),
                                     damping = options.damping_factor(),
                                     ret_iter = True,
                                     weight = graph.use_graph_tool().ep["weight"])

      ret_iter_list.append(ret_iter)
      ranking = graph.ungraph_tool(ranking)
      wsd_rank.set_ranking_for_lemma(analyzed_lemma[0], 
                                     analyzed_lemma[1], 
                                     ranking, 
                                     create_new = False)

    return (wsd_rank, float(sum(ret_iter_list)) / float(len(ret_iter_list)))

  def _set_context_node_set(self, analyzed_lemma, lemma_on_node_dict):
    """!
    Create a set of synsets (precisely nodes from the graph) which belong
    to lemmas from the analyzed context, excluding synsets which belong to
    disambiguated word (lemma).

    @param analyzed_lemma - lemma which is actually disambiguated
    @param lemma_on_node_dict - dictionary where key is (lemma, pos_str) and
                                value is set of synsets (nodes from the graph)
                                belong to this lemma
    """
    self._context_node_set = set()
    for lemma, nodes in lemma_on_node_dict.iteritems():
      if lemma == analyzed_lemma:
        continue
      for node in nodes:
        self._context_node_set.add(node)



class LexicalUnit:
  def __init__(self, lu_id, lemma, pos, domain, variant, lexicon = None):
      self.lu_id = lu_id
      self.lemma = lemma
      self.pos = pos
      self.domain = domain
      self.variant = variant
      self.lexicon = lexicon




from wosedon.lesk_filters.yes import Yes
from wosedon.lesk_filters.no import No
from wosedon.lesk_filters.pos import POS
from wosedon.lesk_filters.words import Words
from wosedon.lesk_filters.wordspos import WordsPOS



#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

from graphbuilderinterface import  GraphBuilderInterface

class GlossGraphBuilder(GraphBuilderInterface):
  def __init__(self, resources, options, str_name = 'GlossGraphBuilder'):
    super(GlossGraphBuilder, self).__init__(resources, options, str_name)



import sys, logging
from wosedon.reranks.rerankinterface import RerankInterface

class LemmaRankingSelecter(RerankInterface):
  """Dla lematow sprawdzana jest roznica pomiedzy wartoscia 
  rankingu pierwszego synsetu, a pozostalymi synsetami. Jezeli
  roznica jest mniejsza niz zadana w pliku konfiguracyjnym
  wartosc percentage_diff, to wybierany jest pomiedzy nimi synset, ktory
  ma najmniejszy wariant."""
  
  def __init__(self, str_name = 'LemmaRankingSelecter'):
    super(LemmaRankingSelecter, self).__init__(str_name)

  def rerank(self, ranking, graph, options):
    logger = logging.getLogger(__name__)
    
    
    tagset = options.tagset()

    new_ranking = []
    for (token, token_rank) in ranking:
      if not token_rank:
        new_ranking.append([token, token_rank])
        continue

      lemma = str(token.get_preferred_lexeme(tagset).lemma())

      rank_sum = sum(rank for (node, rank) in token_rank)
      percentage_diff_value = (float(options.percentage_diff()) * float(rank_sum)) / 100.0
      best_value = token_rank[0][1]

      token_rank_dict = dict(((node, rank), i) for i, (node, rank) in enumerate(token_rank))
      selected_synsets_dict = {}

      for (node, rank), i in token_rank_dict.iteritems():
        diff = abs(best_value - rank)
        if diff <= percentage_diff_value:
          synset_variant = self.get_synset_variant(lemma, node, graph)

          if not synset_variant:
            logger.warning('Lemmas in the synset are completely different from the lemma of the token.')
            continue

          selected_synsets_dict[(node, rank)] = synset_variant

      if len(selected_synsets_dict) == 1:
        new_ranking.append([token, token_rank])
        continue

      new_token_rank = sorted(selected_synsets_dict, key=selected_synsets_dict.get)

      for (node, rank), i in selected_synsets_dict.iteritems():
        del token_rank_dict[(node, rank)]

      new_token_rank.extend(sorted(token_rank_dict, key=token_rank_dict.get))

      new_ranking.append((token, new_token_rank))

    return new_ranking

  def get_synset_variant(self, lemma, node, graph):
    variant = 0
    for lu in node.synset.lu_set:
      if lu.lemma == lemma:
        variant = lu.variant
        break
    return variant



# Implementation of Static pagerank from GraphTool
from wosedon.algorithms.gt_static_pr import GTStaticPR
# Personalized Page Rank imeplementation into graphTool
from wosedon.algorithms.gt_personalized_pr import GTPersonalizedPR
# Personalized Page Rank Word-to-word imeplementation into graphTool
from wosedon.algorithms.gt_personalized_w2w_pr import GTPersonalizedW2WPR
# Normalized PPR into graph tools - only initial vector v is normalized
from wosedon.algorithms.gt_personalized_pr_norm2 import GTPersonalizedPRNorm2

# It looks, that this approach is ctreated by ours
from wosedon.algorithms.gt_personalized_pr_norm import GTPersonalizedPRNorm

# I don't known who is the author of this idea
# Normalized PPR into graph tools - into each loop
from wosedon.algorithms.gt_personalized_pr_norm_it import GTPersonalizedPRNormIt

# Implementation of our ideas for PPR modification
# Normalized PPR with persv modificated into each loop (v = v*Pr)
from wosedon.algorithms.gt_pers_pr_norm_mod_v import GTPersPRNormModV
# Normalized PPR with persv modificated into each loop (v = v*Pr) but
# in each iteration the personalized vector is normalized to range <0,1>
from wosedon.algorithms.gt_pers_pr_norm_it_mod_v import GTPersPRNormItModV
# Normalized PPR with persv modificated into each loop (v = v*Pr) but
# in each iteration the personalized vector is normalized to range <0,1>
# In each loop the ranking is normalized to <0,1> range
from wosedon.algorithms.gt_pers_pr_norm_it_mod_v_rank_norm import GTPersPRNormItModVRankNorm

# usuwanie najgorszego wezla w rankingu - usuwanie az do 50% wartosci rankingu
from wosedon.algorithms.gt_personalized_pr_norm_reduction import GTPersonalizedPRNormReduction

# algorytm Lesk
from wosedon.algorithms.lesk_alg import LeskAlg

from wosedon.algorithms.gt_personalized_w2w_pr_norm import GTPersonalizedW2WPRNorm

from wosedon.algorithms.gt_personalized_pr_norm_two_step import GTPersonalizedPRNormTwoStep

# SUDOKU
from wosedon.algorithms.gt_sudoku_run2 import GTSUDOKURun2

# paintball dla WSD
from wosedon.algorithms.paintball_wsd import PaintballWSD



import unittest
from wosedon.reranks import LemmaRankingNormalizer
from wosedon.basegraph import BaseGraph
from wosedon.utils.config import Config



class TestLemmaRankingNormalizer(unittest.TestCase):
  
  def setUp(self):
    cfg = Config()
    cfg.parse('tests/#Data/wosedon_cosine.ini')
    cfg.resources().set_tagset('nkjp')
    cfg._context = 'Document'
    ctx = cfg.context('tests/#Data/little_ccl.xml', 'tests/#Data/little_ccl.xml')
    
    g = BaseGraph()
    g.init_graph()
    self.nodes = []
    for i in xrange(10):
      self.nodes.append(g.add_node(i))
    
    self.tokens = ctx.next().tokens()
    self.graph = g
    self.rerank = LemmaRankingNormalizer()
    self.options = cfg.rerank_options()
  
  
  def test_it(self):
    ranking = [
      (self.tokens[0], [(self.nodes[0], 3), (self.nodes[1], 2), (self.nodes[2], 1)]),
      (self.tokens[1], [(self.nodes[3], 5)]),
      (self.tokens[2], [(self.nodes[4], 11), (self.nodes[5], 7)]),
      (self.tokens[3], [(self.nodes[6], 13)]),
      (self.tokens[4], [(self.nodes[7], 31), (self.nodes[8], 23), (self.nodes[9], 17)])
    ]
    
    self.assertAlmostEquals(self.rerank.rerank(ranking, self.graph, self.options),
      [
        (self.tokens[0], [(self.nodes[0], 0.5), (self.nodes[1], 0.3333333333333333), (self.nodes[2], 0.16666666666666666)]),
        (self.tokens[1], [(self.nodes[3], 1)]),
        (self.tokens[2], [(self.nodes[4], 0.6111111111111112), (self.nodes[5], 0.3888888888888889)]),
        (self.tokens[3], [(self.nodes[6], 1)]),
        (self.tokens[4], [(self.nodes[7], 0.43661971830985913), (self.nodes[8], 0.323943661971831), (self.nodes[9], 0.23943661971830985)])
      ])
    






SUPPORTED_VALUES = {
    "lemma": None,
    "cat": None,
    "pos": [
        "subst", "depr", "ppron12", "ppron3",
        "siebie", "prep", "num", "intnum",
        "realnum", "intnum-interval",
        "realnum-interval", "symbol", "ordnum",
        "date", "date-interval", "hour-minute", "hour",
        "hour-minute-interval", "hour-interval",
        "year", "year-interval", "day", "day-interval",
        "day-month", "day-month-interval",
        "month-interval", "roman", "roman-interval",
        "roman-ordnum", "match-result",
        "url", "email", "obj-id", "adj", "adjc", "adjp",
        "adja", "adv", "ger", "pact", "ppas", "fin",
        "bedzie", "praet", "winien", "impt", "imps",
        "pred", "aglt", "inf", "pcon", "pant", "qub",
        "comp", "conj", "interj", "sinterj", "burk", "interp",
        "unk"
    ],
    "number": [ "sg", "pl" ],
    "case": [
        "nom", "gen", "dat", "acc", "inst", "loc", "postp", "pred"
    ],
    "gender": [
        "m1", "m2", "m3", "f", "n1", "n2", "p1", "p2", "p3"
    ],
    "person": [
        "pri", "sec", "ter"
    ],
    "grad": [
        "pos", "com", "sup"
    ],
    "praep": [
        "praep", "npraep", "praep-npraep",
    ],
    "acm": [ "congr", "rec" ],
    "ctype": [ "int", "rel", "sub", "coord" ],
    "mode": [ "abl", "adl", "locat", "perl", "dur", "temp", "mod" ],
    "aspect": [ "perf", "imperf" ],
    "negation": [ "neg", "aff" ],
    "mood": [ "indicative", "imperative", "conditional" ],
    "tense": [ "past", "pres", "fut" ],
    "nsyn": [ "proper", "pronoun", "common" ],
    "nsem": [ "count", "time", "mass", "measure" ],
};

def valence_rule(lemma="", pos=""):
    return f"lemma={lemma},pos2={pos},node=concept|sit"

def root_rule():
    return "__root__"

def rule(
    lemma=None,
    pos=None,
    case=None,
    number=None,
    gender=None,
    person=None,
    grad=None,
    praep=None,
    acm=None,
    ctype=None,
    mode=None,
    aspect=None,
    negation=None,
    mood=None,
    tense=None,
    nsyn=None,
    nsem=None
):
    for key in SUPPORTED_VALUES:
        if key in locals():
            val = locals()[key]
            if val is not None:
                if SUPPORTED_VALUES[key] is not None:
                    if val not in SUPPORTED_VALUES[key]:
                        raise Exception(f"Invalid value '{val}' was used for '{key}' parameter. Allowed values are: {', '.join(SUPPORTED_VALUES[key])}")
    output = []
    for key in SUPPORTED_VALUES:
        if key in locals():
            val = locals()[key]
            if val is not None:
                output.append(f"{key}={val}")
    return ",".join(output)


import argparse
import graph_tool
import xml.etree.ElementTree as ET


IN_SENTENCE = 'in_sentence'
IN_PARAGRAPH = 'in_paragraph'
IN_DOCUMENT = 'in_document'


def make_parser():
    desc = 'Extracts graph of synsets from ccl files.'
    parser = argparse.ArgumentParser(description = desc)
    parser.add_argument('-i', help='list of files to process', required=True, metavar='INPUT', nargs='+')
    parser.add_argument('-o', help='output file whither the graph will be stored', default='ccl_graph.xml', metavar='OUTPUT')
    parser.add_argument('-b', help='if set, there will be at most one (best of all) relation between any two nodes', action='store_true', default=False)
    parser.add_argument('-d', help='if set, edges "in_document" will be generated (slow and worthless)', action='store_true', default=False)
    parser.add_argument('-p', help='if set, edges "in_paragraph" will be generated', action='store_true', default=False)
    parser.add_argument('-s', help='if set, edges "in_sentence" will be generated', action='store_true', default=False)
    return parser


def comp_rel(r1, r2):
    """
    Compare two relation types. Return 1 if r2 is better, 0 if they are equal and -1 if r2 is worse.
    """
    if r2 == r1:
        return 0
    if r2 == IN_SENTENCE:
        return 1
    if r1 == IN_SENTENCE:
        return -1
    if r2 == IN_PARAGRAPH:
        return 1
    return -1


def add_edge(graph, s, t, rel, rel_type_prop, only_best):
    if only_best:
        e = graph.edge(s, t)
        if e:
            if comp_rel(rel_type_prop[e], rel) == 1:
                rel_type_prop[e] = rel
            return
  
    e = graph.add_edge(s, t)
    rel_type_prop[e] = rel
  
  

def main(argv = None):
    parser = make_parser()
    args = parser.parse_args(argv)
    
    graph = graph_tool.Graph()
    
    synset_id = graph.new_vertex_property("int")
    graph.vertex_properties['synset_id'] = synset_id
    
    rel_type = graph.new_edge_property("string")
    graph.edge_properties['rel_type'] = rel_type
    
    syn_to_node = {}
    
    counter = 0
    for path in args.i:
        counter += 1
        print "Processing", path, "(" + str(counter) + "/" + str(len(args.i)) + " = " + str(round(counter * 100.0 / len(args.i), 3)) + "%)"
        in_document = set()
        root = ET.parse(path).getroot()
        for chunk in root:
            in_paragraph = set()
            for sentence in chunk:
                in_sentence = set()
                for token in sentence:
                    for prop in token.findall('prop'):
                        if prop.get('key') == "wsd:synset":
                            si = int(prop.text)
                            in_sentence.add(si)
                            in_paragraph.add(si)
                            in_document.add(si)
                            if si not in syn_to_node:
                                node = graph.add_vertex()
                                syn_to_node[si] = node
                                synset_id[node] = si
                if args.s:
                    for syns1 in in_sentence:
                        for syns2 in in_sentence:
                            if syns1 != syns2:
                                add_edge(graph, syn_to_node[syns1], syn_to_node[syns2], IN_SENTENCE, rel_type, args.b)
            
            if args.p:
                for syns1 in in_paragraph:
                    for syns2 in in_paragraph:
                        if syns1 != syns2:
                            add_edge(graph, syn_to_node[syns1], syn_to_node[syns2], IN_PARAGRAPH, rel_type, args.b)
        if args.d:
            for syns1 in in_document:
                for syns2 in in_document:
                    if syns1 != syns2:
                        add_edge(graph, syn_to_node[syns1], syn_to_node[syns2], IN_DOCUMENT, rel_type, args.b)
    
    graph.save(args.o)
                            
                        


if __name__ == '__main__':
        main()



from wosedon.reranks.nodedegreeranker import NodeDegreeRanker
from wosedon.reranks.lemmarankingnormalizer import LemmaRankingNormalizer
from wosedon.reranks.lemmarankingselecter import LemmaRankingSelecter
from wosedon.reranks.lemmarankingfirstselecter import LemmaRankingFirstSelecter

from wosedon.reranks.acontentreranker import AContentReranker


import argparse
import os
import xml.etree.ElementTree as ET


IN_SENTENCE = 'in_sentence'
IN_PARAGRAPH = 'in_paragraph'
IN_DOCUMENT = 'in_document'


def make_parser():
    desc = 'Extracts graph of synsets from ccl files.'
    parser = argparse.ArgumentParser(description = desc)
    parser.add_argument('-f', '--file-list', help='file containing pathes of files to process, relative to both input_dir and input_dir2', required=True)
    parser.add_argument('-i', '--input-dir', help='directory where files with primary rankings are stored', required=True)
    parser.add_argument('-j', '--input-dir2', help='directory where files with secondary rankings are stored', required=True)
    parser.add_argument('-o', '--output-dir', help='directory whither results should be stored', required=True)
    parser.add_argument('-d', '--diff', help='difference from the best to be considered for replacing it, as float', type=float, default=0.3)
    return parser


def make_pair(txt):
    a, b = txt.split('/')
    return int(a), float(b)


def to_resolve(txt, diff):
    txt = txt.split(' ')
    res = set()
    best = make_pair(txt[0])
    for pair in txt:
        pair = make_pair(pair)
        if abs(pair[1] - best[1]) / best[1] < diff:
            res.add(pair[0])
    return res


def help(helper, worthy_synsets):
    res = []
    for prop in helper.findall('prop'):
        if prop.get('key') == 'sense:ukb:syns_rank':
            txt = prop.text.split(' ')
            for pair in txt:
                pair = make_pair(pair)
                if pair[0] in worthy_synsets:
                    res.append(pair)
            return res


def resolve(node, helper, diff):
    for prop in node.findall('prop'):
        if prop.get('key') == 'sense:ukb:syns_rank':
            tr = to_resolve(prop.text, diff)
            if len(tr) > 1:
                newranking = help(helper, tr)
                prop.text = " ".join([str(a) + '/' + str(b) for a,b in newranking])
                for prop2 in node.findall('prop'):
                    if prop2.get('key') == 'sense:ukb:syns_id':
                        prop2.text = str(newranking[0][0])
                        break
                for prop2 in node.findall('prop'):
                    if prop2.get('key') == 'sense:ukb:unitsstr':
                      #node.remove(prop2)
                      prop2.text = ''
                      break
            break
        
  

def main(argv = None):
    parser = make_parser()
    args = parser.parse_args(argv)
    
    file_list = []
    with open(args.file_list) as f:
      for l in f:
        file_list.append(l.strip())
    
    counter = 0
    for path in file_list:
        counter += 1
        print "Processing", path, "(" + str(counter) + "/" + str(len(file_list)) + " = " + str(round(counter * 100.0 / len(file_list), 3)) + "%)"
        
        xml = ET.parse(os.path.join(args.input_dir, path))
        xml2 = ET.parse(os.path.join(args.input_dir2, path))
        
        root = xml.getroot()
        root2 = xml2.getroot()
        
        for chnr in range(len(root)):
            for senr in range(len(root[chnr])):
                for tonr in range(len(root[chnr][senr])):
                    resolve(root[chnr][senr][tonr], root2[chnr][senr][tonr], args.diff)
        
        filename = os.path.join(args.output_dir, path)
        
        if not os.path.exists(os.path.dirname(filename)):
            os.makedirs(os.path.dirname(filename))
        
        xml.write(filename, encoding="utf-8", xml_declaration=True)
        
        
        
                            
                        


if __name__ == '__main__':
        main()



from wosedon.lesk_filters.leskfilterinterface import LeskFilterInterface

class WordsPOS(LeskFilterInterface):
  """!
  Allows only some words with proper part of Speech.
  
  It accepts file with following format:
  
  \code
    word; subst
    home; adj
    home; subst
    tie; *
    *; interp
  \endcode
    
  '*' means anything. "*;*" won't work.'
  """
  
  def __init__(self, tagset, options, str_name='WordsPOS'):
    super(WordsPOS, self).__init__(tagset, options, str_name)
    
    self._allow_only = options.allow_only()
    self._set = set()
    
    if options.input_file():
      with open(options.input_file()) as f:
        for line in f:
          line = line.split(';')
          self._set.add((line[0].strip(), line[1].strip()))
      
  
  def __contains__(self, lexeme):
    lemma = str(lexeme.lemma())
    pos = self.tagset.get_pos_name(lexeme.tag().get_pos_index())
    return self._allow_only == (((lemma, pos) in self._set) or \
                                  (('*', pos) in self._set) or \
                                ((lemma, '*') in self._set))






# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


# This package contains base functions which can are used into wosedon 
# applications. For example base parser for config, model dir and total 
# number of iterations. There is a general method to convert a whole 
# rank to string representation, to adds annotations to CCL files and 
# returns its. An output of annotated CCLs may be returned to stdout or 
# to given filename.

import sys, os, logging
import corpus2
import argparse
import operator

import domains as dom

from basegraph import BaseGraph
try:
    import argcomplete
except ImportError:
    argcomplete = None
    
def make_parser(desc = ''):
  aparser = argparse.ArgumentParser(description = desc)

  aparser.add_argument('-c', '--config-file', dest='config',
                       default='cfg/wosedon.ini',
                       help='WoSeDon config file.')
  aparser.add_argument('-md', '--model-dir', dest='model_dir', 
                       required=False, help=(
                         'Output directory where the models will be stored. '
                         'If is not given then the models will not be stored. '
                         'If output directory is given, and the models '
                         'exists into directory, then tey will be loaded.'))
  aparser.add_argument('-it', '--iterations', dest='iterations', 
                       action='store_true', help=(
                         'Total number of algorithm iterations will be shown '
                         'after WSD algorithm running.'))
  aparser.add_argument('-a', '--alpha', dest='alpha', required=False,
                       help=(
                         'If option is given, then the ranking is '
                         'returned only for these lemmas, where the ranking '
                         'on the first position is a much larger than on '
                         'the second position. The diffrence between first '
                         'and second position is sotred as value of option.'))
  aparser.add_argument('-V', '--verbose', dest='verbose', 
                       action='count',
                       help='If -V is set then warnings will be shown. If it be -VV '
                            'also there will be shown standard informations. Be it '
                            '-VVV even debug output will be shown.')
  if argcomplete:
    argcomplete.autocomplete(aparser)
  return aparser

def make_whole_rank_str(rank):
  '''Converts whole ranking to string like format'''
  return ' '.join('%d/%.10f' % (r[0], r[1]) for r in rank)

def make_unitstr(graph, node):
  fst_unitsstr = ''
  for lu in node.synset.lu_set:
    lu_lemma = str(lu.lemma).replace(' ', '_')
    lu_variant = int(lu.variant)
    lu_domain = int(lu.domain)
    lu_domain_str = dom.DOMAINS[lu_domain] \
      if dom.DOMAINS.has_key(lu_domain) \
      else str(lu_domain)
    manual_synset_str = ('%s.%d(%d:%s) ' % \
                        (lu_lemma, 
                         lu_variant, 
                         lu_domain, 
                         lu_domain_str))
    fst_unitsstr += manual_synset_str
  return fst_unitsstr

def add_annotations_from_rank(ranking, graph, alpha = None):
  for (token, node_rank) in ranking:
    if not node_rank:
      continue

    md_ptr = token.get_metadata()
    if not md_ptr:
      token.create_metadata()
      md_ptr = token.get_metadata()
    
    synset_rank = [(r[0].synset.synset_id, r[1]) 
                   for r in node_rank]

    if len(synset_rank) > 1:
      if alpha:
        diff = synset_rank[0][1] - synset_rank[1][1]
        if (synset_rank[0][1] - synset_rank[1][1]) <= alpha:
          continue

    # best synset ID
    fst_rank_synid = str(synset_rank[0][0])
    # best unit string
    fst_unitsstr = make_unitstr(graph, node_rank[0][0])
    whole_rank_norm_str = make_whole_rank_str(synset_rank)
    md_ptr.set_attribute('sense:ukb:syns_id', fst_rank_synid)
    md_ptr.set_attribute('sense:ukb:syns_rank', whole_rank_norm_str)
    md_ptr.set_attribute('sense:ukb:unitsstr', fst_unitsstr[:-1])

def write_ccl_file(cfg, ccl_document, outccl):
  writer = (
    corpus2.TokenWriter.create_stdout_writer('ccl', cfg.resources().tagset())
      if outccl[0] == '-' \
      else \
        corpus2.TokenWriter.create_path_writer(\
          'ccl', outccl, cfg.resources().tagset()))
  for paragraph in ccl_document.paragraphs():
    writer.write_chunk(paragraph)


def prepare_logging(args):
  loglevel = logging.ERROR
  if args.verbose >= 3:
    loglevel = logging.DEBUG
  elif args.verbose >= 2:
    loglevel = logging.INFO
  elif args.verbose >= 1:
    loglevel = logging.WARNING
  
  class InfoFilter(logging.Filter):
    def filter(self, rec):
      return rec.levelno <= logging.INFO
  
  format_ = "[%(asctime)s]\t%(name)s:%(levelname)s\t%(message)s"
  date_format = "%d %b, %H:%M"
  
  logger = logging.getLogger()
  logger.setLevel(loglevel)
  
  h1 = logging.StreamHandler(sys.stdout)
  h1.setLevel(logging.NOTSET)
  h1.addFilter(InfoFilter())
  
  h2 = logging.StreamHandler(sys.stderr)
  h2.setLevel(logging.WARNING)
  h2.setFormatter(logging.Formatter(format_, date_format))

  logger.addHandler(h1)
  logger.addHandler(h2)


def parse_cfg_to_base_objects(cfg, args=None):
  '''Returns list of objects based on the given config file 
  and application arguments. Parse its and returns the vector with:
  [
   builders,
   mergers,
   wsd_alg,
   list of graphs
  ]
  '''
  prepare_logging(args)
  
  logger = logging.getLogger(__name__)
  
  builders = cfg.gbuilders()
  mergers = cfg.mergers()
  wsd_alg = cfg.wsd_algorithm()

  if len(builders) < 1:
    logger.error('There is not set any builder so graph build is not possible. Have a nice day!')
    exit(1)

  if args and args.model_dir:
    if not os.path.exists(args.model_dir):
      os.makedirs(args.model_dir)

  # makes graphs -> decision about reading/writing from models etc.
  graphs = []
  if len(mergers) > 0:
    if args and args.model_dir:
      gb_filename = "merged_graph.xml.gz"
      gb_file_path = os.path.join(args.model_dir, gb_filename)
      if os.path.exists(gb_file_path):
        logger.info('Loading graph from file %s.', gb_file_path)
        g = BaseGraph()
        g.unpickle(gb_file_path)
      else:
        logger.info('Building graph using mergers.')
        g = builders[0].build_graph()
        for gnum in range(1, len(builders)):
          for mrg in mergers:
            g = mrg.merge(g, builders[gnum].build_graph())
        logger.info('Saving graph to file %s.', gb_file_path)
        g.pickle(gb_file_path)
    else:
      logger.info('Building graph using mergers.')
      g = builders[0].build_graph()
      for gnum in range(1, len(builders)):
        for mrg in mergers:
          g = mrg.merge(g, builders[gnum].build_graph())
    graphs = [g]
  else:
    for gb in builders:
      if args and args.model_dir:
        gb_filename = str(gb) + ".xml.gz"
        gb_file_path = os.path.join(args.model_dir, gb_filename)
        if os.path.exists(gb_file_path):
          logger.info('Loading graph for %s.', str(gb))
          g = BaseGraph()
          g.unpickle(gb_file_path)
        else:
          logger.info('Building graph using %s.', str(gb))
          g = gb.build_graph()
          logger.info('Saving graph to file %s', gb_file_path)
          g.pickle(gb_file_path)
      else:
        logger.info('Building graph using %s.', str(gb))
        g = gb.build_graph()
      graphs.append(g)
      break  # TODO while program cannot use more than one graph there is no need
             # to build them. When status quo changes this 'break' should be removed.
  
  
  logger.info('Setting weights.')
  warned = set()  # lest warn twice
  weights = cfg.weights()
  for g in graphs:
    g.create_edge_attribute('weight', "float", 0.0)
    if weights:
      for e in g.all_edges():
        if e.rel in weights:
          e.weight = weights[e.rel]
        elif e.rel != '' and e.rel not in warned:
          logger.warning('Warning: there is no weight for "%s"! Assuming 0.', e.rel)
          warned.add(e.rel)
    else:
      for e in g.all_edges():
        e.weight = 1.0
  
  logger.info('Building complete.')
  
  return [builders, mergers, wsd_alg, graphs]



import graph_tool as gt
import sys
from random import randint
from collections import deque



def distance(start, end):
    visited = set()
    queue = deque()
    
    visited.add(start)
    queue.append((start, 0))
    
    while queue:
        node, d = queue.popleft()
        if node == end:
            return d
        
        for n in node.out_neighbours():
            if n not in visited:
                visited.add(n)
                queue.append((n, d+1))
    
    return None





print "Loading graph..."
sys.stdout.flush()

graph = gt.load_graph(sys.argv[1])

print "Done.\n"
sys.stdout.flush()

if len(sys.argv) > 2 and sys.argv[2] == 'undirected':
    graph.set_directed(False)
if len(sys.argv) > 2 and sys.argv[2] == 'directed':
    graph.set_directed(True)

s = 0  # sum
n = 0  # number if trials
m = 0  # missing, i.e. how many times there is no way

while True:
    n1 = graph.vertex(randint(0, graph.num_vertices()-1))
    n2 = graph.vertex(randint(0, graph.num_vertices()-1))
    
    d = distance(n1, n2)
    
    n += 1
    
    if d == None:
        m += 1
    else:
        s += d
    
    print 'trial: ' + str(n) + ', average lenght: ' + str(s / float(n-m)) + ', unconnected: ' + str(m * 100.0 / n) + '%'
    sys.stdout.flush()
    
    
    
